# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

# Imports ###########################################################

import json
import logging
from collections import namedtuple
from decimal import ROUND_HALF_UP, Decimal
from itertools import chain

import six
from lazy.lazy import lazy
from xblock.core import XBlock
from xblock.exceptions import JsonHandlerError, NoSuchViewError
from xblock.fields import Boolean, Float, Integer, List, Scope, String
from xblock.fragment import Fragment
from xblock.validation import ValidationMessage
from xblockutils.helpers import child_isinstance
from xblockutils.resources import ResourceLoader
from xblockutils.settings import XBlockWithSettingsMixin
from xblockutils.studio_editable import (NestedXBlockSpec,
                                         StudioContainerWithNestedXBlocksMixin,
                                         StudioEditableXBlockMixin)

from problem_builder.answer import AnswerBlock, AnswerRecapBlock
from problem_builder.completion import CompletionBlock
from problem_builder.mcq import MCQBlock, RatingBlock
from problem_builder.mrq import MRQBlock
from problem_builder.plot import PlotBlock
from problem_builder.slider import SliderBlock
from problem_builder.swipe import SwipeBlock
from problem_builder.table import MentoringTableBlock

from .message import MentoringMessageBlock, get_message_label
from .mixins import (ExpandStaticURLMixin, MessageParentMixin, QuestionMixin,
                     StepParentMixin, StudentViewUserStateMixin,
                     StudentViewUserStateResultsTransformerMixin,
                     XBlockWithTranslationServiceMixin, _normalize_id)
from .step_review import ReviewStepBlock
from .utils import I18NService

try:
    # Used to detect if we're in the workbench so we can add Font Awesome
    from workbench.runtime import WorkbenchRuntime
except ImportError:
    WorkbenchRuntime = False

# Globals ###########################################################

log = logging.getLogger(__name__)
loader = ResourceLoader(__name__)

_default_options_config = {
    'pb_mcq_hide_previous_answer': False,  # this works for both MCQs and MRQs.
    'pb_hide_feedback_if_attempts_remain': False,
}


# Make '_' a no-op so we can scrape strings
def _(text):
    return text

# Classes ###########################################################


Score = namedtuple("Score", ["raw", "percentage", "correct", "incorrect", "partially_correct"])

CORRECT = 'correct'
INCORRECT = 'incorrect'
PARTIAL = 'partial'


@XBlock.needs("i18n")
@XBlock.wants('settings')
class BaseMentoringBlock(
    XBlock, XBlockWithTranslationServiceMixin, XBlockWithSettingsMixin,
    StudioEditableXBlockMixin, MessageParentMixin, StudentViewUserStateMixin,
    ExpandStaticURLMixin
):
    """
    An XBlock that defines functionality shared by mentoring blocks.
    """
    # Content
    show_title = Boolean(
        display_name=_("Show title"),
        help=_("Display the title?"),
        default=True,
        scope=Scope.content
    )
    max_attempts = Integer(
        display_name=_("Max. attempts allowed"),
        help=_("Maximum number of times students are allowed to attempt the questions belonging to this block"),
        default=0,
        scope=Scope.content,
        enforce_type=True
    )
    weight = Float(
        display_name=_("Weight"),
        help=_("Defines the maximum total grade of the block."),
        default=1,
        scope=Scope.settings,
        enforce_type=True
    )

    # User state
    num_attempts = Integer(
        # Number of attempts a user has answered for this questions
        default=0,
        scope=Scope.user_state,
        enforce_type=True
    )

    has_children = True
    has_score = True  # The Problem/Step Builder XBlocks produce scores. (Their children do not send scores to the LMS.)

    icon_class = 'problem'
    block_settings_key = 'mentoring'
    options_key = 'options'

    @property
    def url_name(self):
        """
        Get the url_name for this block. In Studio/LMS it is provided by a mixin, so we just
        defer to super(). In the workbench or any other platform, we use the usage_id.
        """
        try:
            return super(BaseMentoringBlock, self).url_name
        except AttributeError:
            return six.text_type(self.scope_ids.usage_id)

    @property
    def review_tips_json(self):
        return json.dumps(self.review_tips)

    @property
    def max_attempts_reached(self):
        return self.max_attempts > 0 and self.num_attempts >= self.max_attempts

    def get_content_titles(self):
        """
        By default, each Sequential block in a course ("Subsection" in Studio parlance) will
        display the display_name of each descendant in a tooltip above the content. We don't
        want that - we only want to display one title for this mentoring block as a whole.
        Otherwise things like "Choice (yes) (Correct)" will appear in the tooltip.

        If this block has no title set, don't display any title. Then, if this is the only block
        in the unit, the unit's title will be used. (Why isn't it always just used?)
        """
        has_explicitly_set_title = self.fields['display_name'].is_set_on(self)
        if has_explicitly_set_title:
            return [self.display_name]
        return []

    def get_options(self):
        """
        Get options settings for this block from settings service.

        Fall back on default options if xblock settings have not been customized at all
        or no customizations for options available.
        """
        xblock_settings = self.get_xblock_settings(default={})
        if xblock_settings and self.options_key in xblock_settings:
            return xblock_settings[self.options_key]
        return _default_options_config

    def get_option(self, option):
        """
        Get value of a specific instance-wide `option`.
        """
        return self.get_options().get(option)

    @XBlock.json_handler
    def view(self, data, suffix=''):
        """
        Current HTML view of the XBlock, for refresh by client
        """
        frag = self.student_view({})
        return {'html': frag.content}

    @XBlock.json_handler
    def publish_event(self, data, suffix=''):
        """
        Publish data for analytics purposes
        """
        event_type = data.pop('event_type')
        if (event_type == 'grade'):
            # This handler can be called from the browser. Don't allow the browser to submit arbitrary grades ;-)
            raise JsonHandlerError(403, "Posting grade events from the browser is forbidden.")

        self.runtime.publish(self, event_type, data)
        return {'result': 'ok'}

    def author_preview_view(self, context):
        """
        Child blocks can override this to add a custom preview shown to
        authors in Studio when not editing this block's children.
        """
        fragment = self.student_view(context)
        fragment.add_content(loader.render_template('templates/html/mentoring_url_name.html', {
            "url_name": self.url_name
        }))
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder-edit.css'))
        return fragment

    def max_score(self):
        """ Maximum score. We scale all scores to a maximum of 1.0 so this is always 1.0 """
        return 1.0


class MentoringBlock(
    StudentViewUserStateResultsTransformerMixin, I18NService,
    BaseMentoringBlock, StudioContainerWithNestedXBlocksMixin, StepParentMixin,
):
    """
    An XBlock providing mentoring capabilities

    Composed of text, answers input fields, and a set of MRQ/MCQ with advices.
    A set of conditions on the provided answers and MCQ/MRQ choices will determine if the
    student is a) provided mentoring advices and asked to alter his answer, or b) is given the
    ok to continue.
    """
    # Content
    USER_STATE_FIELDS = ['completed', 'num_attempts', 'student_results']
    followed_by = String(
        display_name=_("Followed by"),
        help=_("url_name of the step after the current mentoring block in workflow."),
        default=None,
        scope=Scope.content
    )
    enforce_dependency = Boolean(
        display_name=_("Enforce Dependency"),
        help=_("Should the next step be the current block to complete?"),
        default=False,
        scope=Scope.content,
        enforce_type=True
    )
    display_submit = Boolean(
        display_name=_("Show Submit Button"),
        help=_("Allow submission of the current block?"),
        default=True,
        scope=Scope.content,
        enforce_type=True
    )
    xml_content = String(
        display_name=_("XML content"),
        help=_("Not used for version 2. This field is here only to preserve the data needed to upgrade from v1 to v2."),
        default='',
        scope=Scope.content,
        multiline_editor=True
    )

    # Settings
    display_name = String(
        display_name=_("Title (Display name)"),
        help=_("Title to display"),
        default=_("Problem Builder"),
        scope=Scope.settings
    )
    feedback_label = String(
        display_name=_("Feedback Header"),
        help=_("Header for feedback messages"),
        default=_("Feedback"),
        scope=Scope.content
    )

    # User state
    attempted = Boolean(
        # Has the student attempted this mentoring step?
        default=False,
        scope=Scope.user_state
        # TODO: Does anything use this 'attempted' field? May want to delete it.
    )
    completed = Boolean(
        # Has the student completed this mentoring step?
        default=False,
        scope=Scope.user_state
    )
    step = Integer(
        # Keep track of the student assessment progress.
        default=0,
        scope=Scope.user_state,
        enforce_type=True
    )
    student_results = List(
        # Store results of student choices.
        default=[],
        scope=Scope.user_state
    )
    extended_feedback = Boolean(
        help=_("Show extended feedback details when all attempts are used up."),
        default=False,
        Scope=Scope.content
    )

    # Global user state
    next_step = String(
        # url_name of the next step the student must complete (global to all blocks)
        default='mentoring_first',
        scope=Scope.preferences
    )

    editable_fields = (
        'display_name', 'followed_by', 'max_attempts', 'enforce_dependency',
        'display_submit', 'feedback_label', 'weight', 'extended_feedback'
    )

    @property
    def allowed_nested_blocks(self):
        """
        Returns a list of allowed nested XBlocks. Each item can be either
        * An XBlock class
        * A NestedXBlockSpec

        If XBlock class is used it is assumed that this XBlock is enabled and allows multiple instances.
        NestedXBlockSpec allows explicitly setting disabled/enabled state, disabled reason (if any) and single/multiple
        instances
        """
        additional_blocks = []
        try:
            from xmodule.video_module.video_module import VideoDescriptor
            additional_blocks.append(NestedXBlockSpec(
                VideoDescriptor, category='video', label=_(u"Video")
            ))
        except ImportError:
            pass
        try:
            from imagemodal import ImageModal
            additional_blocks.append(NestedXBlockSpec(
                ImageModal, category='imagemodal', label=_(u"Image Modal")
            ))
        except ImportError:
            pass

        try:
            from xblock_django.models import XBlockConfiguration
            opt = XBlockConfiguration.objects.filter(name="pb-swipe")
            if opt.count() and opt.first().enabled:
                additional_blocks.append(SwipeBlock)
        except ImportError:
            pass

        try:
            from ooyala_player import OoyalaPlayerBlock
            additional_blocks.append(NestedXBlockSpec(
                OoyalaPlayerBlock, category='ooyala-player', label=_(u"Ooyala Player")
            ))
        except ImportError:
            pass

        message_block_shims = [
            NestedXBlockSpec(
                MentoringMessageBlock,
                category='pb-message',
                boilerplate=message_type,
                label=get_message_label(message_type),
            )
            for message_type in (
                'completed',
                'incomplete',
                'max_attempts_reached',
            )
        ]

        return [
            NestedXBlockSpec(AnswerBlock, boilerplate='studio_default'),
            MCQBlock, RatingBlock, MRQBlock, CompletionBlock,
            NestedXBlockSpec(None, category="html", label=self._("HTML")),
            AnswerRecapBlock, MentoringTableBlock, PlotBlock, SliderBlock
        ] + additional_blocks + message_block_shims

    def get_question_number(self, question_id):
        """
        Get the step number of the question id
        """
        for child_id in self.children:
            question = self.runtime.get_block(child_id)
            if isinstance(question, QuestionMixin) and (question.name == question_id):
                return question.step_number
        raise ValueError("Question ID in answer set not a step of this Mentoring Block!")

    def answer_mapper(self, answer_status):
        """
        Create a JSON-dumpable object with readable key names from a list of student answers.
        """
        answer_map = []
        for answer in self.student_results:
            if answer[1]['status'] == answer_status:
                try:
                    answer_map.append({
                        'number': self.get_question_number(answer[0]),
                        'id': answer[0],
                        'details': answer[1],
                    })
                except ValueError:
                    pass  # The question has been deleted since the student answered it.
        return answer_map

    @property
    def score(self):
        """Compute the student score taking into account the weight of each step."""
        steps = self.steps
        steps_map = {q.name: q for q in steps}
        total_child_weight = sum(float(step.weight) for step in steps)
        if total_child_weight == 0:
            return Score(0, 0, [], [], [])
        points_earned = 0
        for q_name, q_details in self.student_results:
            question = steps_map.get(q_name)
            if question:
                points_earned += q_details['score'] * question.weight
        score = Decimal(points_earned) / Decimal(total_child_weight)
        correct = self.answer_mapper(CORRECT)
        incorrect = self.answer_mapper(INCORRECT)
        partially_correct = self.answer_mapper(PARTIAL)

        return Score(
            float(score),
            int(Decimal(score * 100).quantize(Decimal('1.'), rounding=ROUND_HALF_UP)),
            correct,
            incorrect,
            partially_correct
        )

    @XBlock.supports("multi_device")  # Mark as mobile-friendly
    def student_view(self, context):
        from .questionnaire import QuestionnaireAbstractBlock  # Import here to avoid circular dependency

        # Migrate stored data if necessary
        self.migrate_fields()

        # Validate self.step:
        num_steps = len(self.steps)
        if self.step > num_steps:
            self.step = num_steps

        fragment = Fragment()
        child_content = u""

        mcq_hide_previous_answer = self.get_option('pb_mcq_hide_previous_answer')

        for child_id in self.children:
            child = self.runtime.get_block(child_id)
            if child is None:  # child should not be None but it can happen due to bugs or permission issues
                child_content += u"<p>[{}]</p>".format(self._(u"Error: Unable to load child component."))
            elif not isinstance(child, MentoringMessageBlock):
                try:
                    if mcq_hide_previous_answer and isinstance(child, QuestionnaireAbstractBlock):
                        context['hide_prev_answer'] = True
                    else:
                        context['hide_prev_answer'] = False
                    child_fragment = child.render('mentoring_view', context)
                except NoSuchViewError:
                    if child.scope_ids.block_type == 'html' and getattr(self.runtime, 'is_author_mode', False):
                        # html block doesn't support mentoring_view, and if we use student_view Studio will wrap
                        # it in HTML that we don't want in the preview. So just render its HTML directly:
                        child_fragment = Fragment(child.data)
                    else:
                        child_fragment = child.render('student_view', context)
                fragment.add_frag_resources(child_fragment)
                child_content += child_fragment.content

        fragment.add_content(loader.render_django_template('templates/html/mentoring.html', {
            'self': self,
            'title': self.display_name,
            'show_title': self.show_title,
            'child_content': child_content,
            'missing_dependency_url': self.has_missing_dependency and self.next_step_url,
        }, i18n_service=self.i18n_service))
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder.css'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/vendor/underscore-min.js'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/util.js'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/mentoring_standard_view.js'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/mentoring.js'))
        fragment.add_resource(loader.load_unicode('templates/html/mentoring_attempts.underscore'), "text/html")

        # Workbench doesn't have font awesome, so add it:
        if WorkbenchRuntime and isinstance(self.runtime, WorkbenchRuntime):
            fragment.add_css_url('//maxcdn.bootstrapcdn.com/font-awesome/4.3.0/css/font-awesome.min.css')

        fragment.initialize_js('MentoringBlock')

        if not self.display_submit:
            self.runtime.publish(self, 'progress', {})

        return fragment

    def migrate_fields(self):
        """
        Migrate data stored in the fields, when a format change breaks backward-compatibility with
        previous data formats
        """
        # Partial answers replaced the `completed` with `status` in `self.student_results`
        if self.student_results and 'completed' in self.student_results[0][1]:
            # Rename the field and use the new value format (text instead of boolean)
            for result in self.student_results:
                result[1]['status'] = 'correct' if result[1]['completed'] else 'incorrect'
                del result[1]['completed']

    @property
    def additional_publish_event_data(self):
        return {
            'user_id': self.scope_ids.user_id,
            'component_id': self.url_name,
        }

    @property
    def has_missing_dependency(self):
        """
        Returns True if the student needs to complete another step before being able to complete
        the current one, and False otherwise
        """
        return self.enforce_dependency and (not self.completed) and (self.next_step != self.url_name)

    @property
    def next_step_url(self):
        """
        Returns the URL of the next step's page
        """
        return '/jump_to_id/{}'.format(self.next_step)

    @property
    def hide_feedback(self):
        return self.get_option("pb_hide_feedback_if_attempts_remain") and not self.max_attempts_reached

    def get_message(self, completed):
        """
        Get the message to display to a student following a submission.
        """
        if completed:
            # Student has achieved a perfect score
            return self.get_message_content('completed')
        elif self.max_attempts_reached:
            # Student has not achieved a perfect score and cannot try again
            return self.get_message_content('max_attempts_reached')
        else:
            # Student did not achieve a perfect score but can try again:
            return self.get_message_content('incomplete')

    @property
    def review_tips(self):
        review_tips = []
        return review_tips

    def show_extended_feedback(self):
        return self.extended_feedback and self.max_attempts_reached

    @XBlock.json_handler
    def get_results(self, queries, suffix=''):
        """
        Gets detailed results in the case of extended feedback.

        Right now there are two ways to get results-- through the template upon loading up
        the mentoring block, or after submission of an AJAX request like in
        submit or get_results here.
        """
        results, completed, show_message = self._get_standard_results()
        mentoring_completed = completed

        result = {
            'results': results,
            'completed': completed,
            'step': self.step,
            'max_attempts': self.max_attempts,
            'num_attempts': self.num_attempts,
        }

        if show_message:
            result['message'] = self.get_message(mentoring_completed)

        return result

    def _get_standard_results(self):
        """
        Gets previous submissions results as if submit was called with exactly the same values as last time.
        """
        results = []
        completed = True
        show_message = (not self.hide_feedback) and bool(self.student_results)

        # All children are visible simultaneously, so need to collect results for all of them
        for child in self.steps:
            child_result = child.get_last_result()
            results.append([child.name, child_result])
            completed = completed and (child_result.get('status', None) == 'correct')

        return results, completed, show_message

    @XBlock.json_handler
    def submit(self, submissions, suffix=''):
        log.info(u'Received submissions: {}'.format(submissions))
        # server-side check that the user is allowed to submit:
        if self.max_attempts_reached:
            raise JsonHandlerError(403, "Maximum number of attempts already reached.")
        elif self.has_missing_dependency:
            raise JsonHandlerError(
                403,
                "You need to complete all previous steps before being able to complete the current one."
            )

        # This has now been attempted:
        self.attempted = True

        submit_results = []
        previously_completed = self.completed
        completed = True
        for child in self.steps:
            if child.name and child.name in submissions:
                submission = submissions[child.name]
                child_result = child.submit(submission)
                submit_results.append([child.name, child_result])
                child.save()
                completed = completed and (child_result['status'] == 'correct')

        if completed and self.next_step == self.url_name:
            self.next_step = self.followed_by

        # Update the score and attempts, unless the user had already achieved a perfect score ("completed"):
        if not previously_completed:
            # Update the results
            while self.student_results:
                self.student_results.pop()
            for result in submit_results:
                self.student_results.append(result)

            # Save the user's latest score
            self.runtime.publish(self, 'grade', {
                'value': self.score.raw,
                'max_value': self.max_score(),
            })

            # Mark this as having used an attempt:
            if self.max_attempts > 0:
                self.num_attempts += 1

        # Save the completion status.
        # Once it has been completed once, keep completion even if user changes values
        self.completed = bool(completed) or previously_completed

        message = self.get_message(completed)
        raw_score = self.score.raw

        self.runtime.publish(self, 'xblock.problem_builder.submitted', {
            'num_attempts': self.num_attempts,
            'submitted_answer': submissions,
            'grade': raw_score,
        })

        return {
            'results': submit_results,
            'completed': self.completed,
            'message': message,
            'max_attempts': self.max_attempts,
            'num_attempts': self.num_attempts,
        }

    def feedback_dispatch(self, target_data, stringify):
        if self.show_extended_feedback():
            if stringify:
                return json.dumps(target_data)
            else:
                return target_data

    def correct_json(self, stringify=True):
        return self.feedback_dispatch(self.score.correct, stringify)

    def incorrect_json(self, stringify=True):
        return self.feedback_dispatch(self.score.incorrect, stringify)

    def partial_json(self, stringify=True):
        return self.feedback_dispatch(self.score.partially_correct, stringify)

    @XBlock.json_handler
    def try_again(self, data, suffix=''):

        if self.max_attempts_reached:
            return {
                'result': 'error',
                'message': 'max attempts reached'
            }

        # reset
        self.step = 0
        self.completed = False

        while self.student_results:
            self.student_results.pop()

        return {
            'result': 'success'
        }

    def validate(self):
        """
        Validates the state of this XBlock except for individual field values.
        """
        validation = super(MentoringBlock, self).validate()
        a_child_has_issues = False
        message_types_present = set()
        for child_id in self.children:
            child = self.runtime.get_block(child_id)
            # Check if the child has any errors:
            if not child.validate().empty:
                a_child_has_issues = True
            # Ensure there is only one "message" block of each type:
            if isinstance(child, MentoringMessageBlock):
                msg_type = child.type
                if msg_type in message_types_present:
                    validation.add(ValidationMessage(
                        ValidationMessage.ERROR,
                        self._(u"There should only be one '{msg_type}' message component.").format(msg_type=msg_type)
                    ))
                message_types_present.add(msg_type)
        if a_child_has_issues:
            validation.add(ValidationMessage(
                ValidationMessage.ERROR,
                self._(u"A component inside this mentoring block has issues.")
            ))
        return validation

    def author_edit_view(self, context):
        """
        Add some HTML to the author view that allows authors to add child blocks.
        """
        local_context = context.copy()
        local_context['author_edit_view'] = True
        fragment = super(MentoringBlock, self).author_edit_view(local_context)
        fragment.add_content(loader.render_template('templates/html/mentoring_url_name.html', {
            'url_name': self.url_name
        }))
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder.css'))
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder-edit.css'))
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder-tinymce-content.css'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/util.js'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/container_edit.js'))
        fragment.initialize_js('ProblemBuilderContainerEdit')

        return fragment

    @staticmethod
    def workbench_scenarios():
        """
        Scenarios displayed by the workbench. Load them from external (private) repository
        """
        return loader.load_scenarios_from_path('templates/xml')

    def student_view_data(self, context=None):
        """
        Returns a JSON representation of the student_view of this XBlock,
        retrievable from the Course Block API.
        """
        components = []
        for child_id in self.children:
            block = self.runtime.get_block(child_id)
            if hasattr(block, 'student_view_data'):
                components.append(block.student_view_data())

        return {
            'block_id': six.text_type(self.scope_ids.usage_id),
            'display_name': self.display_name,
            'max_attempts': self.max_attempts,
            'extended_feedback': self.extended_feedback,
            'feedback_label': self.feedback_label,
            'components': components,
            'messages': {
                message_type: self.expand_static_url(self.get_message_content(message_type))
                for message_type in (
                        'completed',
                        'incomplete',
                        'max_attempts_reached',
                )
            }
        }


class MentoringWithExplicitStepsBlock(BaseMentoringBlock, StudioContainerWithNestedXBlocksMixin,
                                      I18NService):
    """
    An XBlock providing mentoring capabilities with explicit steps
    """
    USER_STATE_FIELDS = ['num_attempts']

    # Content
    extended_feedback = Boolean(
        display_name=_("Extended feedback"),
        help=_("Show extended feedback when all attempts are used up?"),
        default=False,
        Scope=Scope.content
    )

    # Settings
    display_name = String(
        display_name=_("Title (display name)"),
        help=_("Title to display"),
        default=_("Step Builder"),
        scope=Scope.settings
    )

    # User state
    active_step = Integer(
        # Keep track of the student progress.
        default=0,
        scope=Scope.user_state,
        enforce_type=True
    )

    editable_fields = ('display_name', 'max_attempts', 'extended_feedback', 'weight')

    def build_user_state_data(self, context=None):
        user_state_data = super(MentoringWithExplicitStepsBlock, self).build_user_state_data()
        user_state_data['active_step'] = self.active_step_safe
        user_state_data['score_summary'] = self.get_score_summary()
        return user_state_data

    @lazy
    def question_ids(self):
        """
        Get the usage_ids of all of this XBlock's children that are "Questions".
        """
        return list(chain.from_iterable(self.runtime.get_block(step_id).step_ids for step_id in self.step_ids))

    @lazy
    def questions(self):
        """
        Get all questions associated with this block.
        """
        return [self.runtime.get_block(question_id) for question_id in self.question_ids]

    @property
    def active_step_safe(self):
        """
        Get self.active_step and double-check that it is a valid value.
        The stored value could be invalid if this block has been edited and new steps were
        added/deleted.
        """
        active_step = self.active_step
        if active_step >= 0 and active_step < len(self.step_ids):
            return active_step
        if active_step == -1 and self.has_review_step:
            return active_step  # -1 indicates the review step
        return 0

    def get_active_step(self):
        """ Get the active step as an instantiated XBlock """
        block = self.runtime.get_block(self.step_ids[self.active_step_safe])
        if block is None:
            log.error("Unable to load step builder step child %s", self.step_ids[self.active_step_safe])
        return block

    @lazy
    def step_ids(self):
        """
        Get the usage_ids of all of this XBlock's children that are steps.
        """
        from .step import MentoringStepBlock  # Import here to avoid circular dependency
        return [
            _normalize_id(child_id) for child_id in self.children if
            child_isinstance(self, child_id, MentoringStepBlock)
        ]

    @lazy
    def steps(self):
        """
        Get the step children of this block.
        """
        return [self.runtime.get_block(step_id) for step_id in self.step_ids]

    def get_question_number(self, question_name):
        question_names = [q.name for q in self.questions]
        return question_names.index(question_name) + 1

    def answer_mapper(self, answer_status):
        steps = self.steps
        answer_map = []
        for step in steps:
            for answer in step.student_results:
                if answer[1]['status'] == answer_status:
                    answer_map.append({
                        'id': answer[0],
                        'details': answer[1],
                        'step': step.step_number,
                        'number': self.get_question_number(answer[0]),
                    })
        return answer_map

    @property
    def has_review_step(self):
        return any(child_isinstance(self, child_id, ReviewStepBlock) for child_id in self.children)

    @property
    def review_step(self):
        """ Get the Review Step XBlock child, if any. Otherwise returns None """
        for step_id in self.children:
            if child_isinstance(self, step_id, ReviewStepBlock):
                return self.runtime.get_block(step_id)

    @property
    def score(self):
        questions = self.questions
        total_child_weight = sum(float(question.weight) for question in questions)
        if total_child_weight == 0:
            return Score(0, 0, [], [], [])
        steps = self.steps
        questions_map = {question.name: question for question in questions}
        points_earned = 0
        for step in steps:
            for question_name, question_results in step.student_results:
                question = questions_map.get(question_name)
                if question:  # Under what conditions would this evaluate to False?
                    points_earned += question_results['score'] * question.weight
        score = Decimal(points_earned) / Decimal(total_child_weight)
        correct = self.answer_mapper(CORRECT)
        incorrect = self.answer_mapper(INCORRECT)
        partially_correct = self.answer_mapper(PARTIAL)

        return Score(
            float(score),
            int((Decimal(score) * 100).quantize(Decimal('1.'), rounding=ROUND_HALF_UP)),
            correct,
            incorrect,
            partially_correct
        )

    @property
    def complete(self):
        return not self.score.incorrect and not self.score.partially_correct

    @property
    def review_tips(self):
        """ Get review tips, shown for wrong answers. """
        if self.max_attempts > 0 and self.num_attempts >= self.max_attempts:
            # Review tips are only shown if the student is allowed to try again.
            return []
        review_tips = []
        status_cache = dict()
        steps = self.steps
        for step in steps:
            status_cache.update(dict(step.student_results))
        for question in self.questions:
            result = status_cache.get(question.name)
            if result and result.get('status') != 'correct':
                # The student got this wrong. Check if there is a review tip to show.
                tip_html = question.get_review_tip()
                if tip_html:
                    if getattr(self.runtime, 'replace_jump_to_id_urls', None) is not None:
                        tip_html = self.runtime.replace_jump_to_id_urls(tip_html)
                    review_tips.append(tip_html)
        return review_tips

    def show_extended_feedback(self):
        return self.extended_feedback and self.max_attempts_reached

    @XBlock.supports("multi_device")  # Mark as mobile-friendly
    def student_view(self, context):
        fragment = Fragment()
        children_contents = []

        context = context or {}
        context['hide_prev_answer'] = True  # For Step Builder, we don't show the users' old answers when they try again
        context['score_summary'] = self.get_score_summary()
        for child_id in self.children:
            child = self.runtime.get_block(child_id)
            if child is None:  # child should not be None but it can happen due to bugs or permission issues
                child_content = u"<p>[{}]</p>".format(self._(u"Error: Unable to load child component."))
            else:
                child_fragment = self._render_child_fragment(child, context, view='mentoring_view')
                fragment.add_frag_resources(child_fragment)
                child_content = child_fragment.content
            children_contents.append(child_content)

        fragment.add_content(loader.render_django_template('templates/html/mentoring_with_steps.html', {
            'self': self,
            'title': self.display_name,
            'show_title': self.show_title,
            'children_contents': children_contents,
        }, i18n_service=self.i18n_service))
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder.css'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/vendor/underscore-min.js'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/step_util.js'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/mentoring_with_steps.js'))

        fragment.add_resource(loader.load_unicode('templates/html/mentoring_attempts.underscore'), "text/html")
        fragment.initialize_js('MentoringWithStepsBlock', {
            'show_extended_feedback': self.show_extended_feedback(),
        })

        return fragment

    @property
    def allowed_nested_blocks(self):
        """
        Returns a list of allowed nested XBlocks. Each item can be either
        * An XBlock class
        * A NestedXBlockSpec

        If XBlock class is used it is assumed that this XBlock is enabled and allows multiple instances.
        NestedXBlockSpec allows explicitly setting disabled/enabled state, disabled reason (if any) and single/multiple
        instances
        """
        # Import here to avoid circular dependency
        from .step import MentoringStepBlock
        return [
            MentoringStepBlock,
            NestedXBlockSpec(ReviewStepBlock, single_instance=True),
        ]

    @XBlock.json_handler
    def submit(self, data, suffix=None):
        """
        Called when the user has submitted the answer[s] for the current step.
        """
        # First verify that active_step is correct:
        if data.get("active_step") != self.active_step_safe:
            raise JsonHandlerError(400, "Invalid Step. Refresh the page and try again.")

        # The step child will process the data:
        step_block = self.get_active_step()
        if not step_block:
            raise JsonHandlerError(500, "Unable to load the current step block.")
        response_data = step_block.submit(data)

        # Update the active step:
        new_value = self.active_step_safe + 1
        if new_value < len(self.step_ids):
            self.active_step = new_value
        elif new_value == len(self.step_ids):
            # The user just completed the final step.
            # Update the number of attempts:
            self.num_attempts += 1
            # Do we need to render a review (summary of the user's score):
            if self.has_review_step:
                self.active_step = -1
                response_data['review_html'] = self.runtime.render(self.review_step, "mentoring_view", {
                    'score_summary': self.get_score_summary(),
                }).content
            response_data['num_attempts'] = self.num_attempts
            # And publish the score:
            score = self.score
            grade_data = {
                'value': score.raw,
                'max_value': self.max_score(),
            }
            self.runtime.publish(self, 'grade', grade_data)

        response_data['active_step'] = self.active_step
        return response_data

    def get_score_summary(self):
        if self.num_attempts == 0:
            return {}
        score = self.score
        return {
            'score': score.percentage,
            'correct_answers': len(score.correct),
            'incorrect_answers': len(score.incorrect),
            'partially_correct_answers': len(score.partially_correct),
            'correct': score.correct,
            'incorrect': score.incorrect,
            'partial': score.partially_correct,
            'complete': self.complete,
            'max_attempts_reached': self.max_attempts_reached,
            'show_extended_review': self.show_extended_feedback(),
            'review_tips': self.review_tips,
        }

    @XBlock.json_handler
    def get_num_attempts(self, data, suffix):
        return {
            'num_attempts': self.num_attempts
        }

    @XBlock.json_handler
    def try_again(self, data, suffix=''):
        self.active_step = 0

        step_blocks = [self.runtime.get_block(child_id) for child_id in self.step_ids]

        for step in step_blocks:
            step.reset()

        return {
            'active_step': self.active_step
        }

    def author_preview_view(self, context):
        return self.student_view(context)

    def author_edit_view(self, context):
        """
        Add some HTML to the author view that allows authors to add child blocks.
        """
        fragment = super(MentoringWithExplicitStepsBlock, self).author_edit_view(context)
        fragment.add_content(loader.render_template('templates/html/mentoring_url_name.html', {
            "url_name": self.url_name
        }))
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder.css'))
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder-edit.css'))
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder-tinymce-content.css'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/util.js'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/container_edit.js'))
        fragment.initialize_js('ProblemBuilderContainerEdit')
        return fragment

    def student_view_data(self, context=None):
        components = []

        for child_id in self.children:
            child = self.runtime.get_block(child_id)
            if hasattr(child, 'student_view_data'):
                components.append(child.student_view_data(context))

        return {
            'title': self.display_name,
            'block_id': six.text_type(self.scope_ids.usage_id),
            'display_name': self.display_name,
            'show_title': self.show_title,
            'weight': self.weight,
            'extended_feedback': self.extended_feedback,
            'max_attempts': self.max_attempts,
            'components': components,
        }



"""
Helper methods for testing Problem Builder / Step Builder blocks
"""
import json
from datetime import date, datetime

from mock import MagicMock, Mock, patch
from xblock.field_data import DictFieldData


class ScoresTestMixin(object):
    """
    Mixin for tests that involve scores (grades)
    """
    def assert_produces_scores(self, block):
        """
        Test that the given XBlock instance meets the requirements of being able to report
        scores to the edX LMS, and have them appear on the student's progress page.
        """
        self.assertTrue(block.has_score)
        self.assertTrue(type(block).has_score)
        self.assertEqual(block.weight, 1.0)  # Default weight should be 1
        self.assertIsInstance(block.max_score(), (int, float))


class BlockWithChildrenTestMixin(object):
    """
    Mixin for tests targeting blocks that contain nested child blocks.
    """

    ALLOWED_NESTED_BLOCKS = [
        'pb-answer',
        'pb-mcq',
        'pb-rating',
        'pb-mrq',
        'pb-completion',
        'html',
        'pb-answer-recap',
        'pb-table',
        'sb-plot',
        'pb-slider',
    ]

    ADDITIONAL_BLOCKS = [
        'video',
        'imagemodal',
        'ooyala-player',
    ]

    def get_allowed_blocks(self, block):
        """
        Return list of categories corresponding to child blocks allowed for `block`.
        """
        return [
            getattr(allowed_block, 'category', getattr(allowed_block, 'CATEGORY', None))
            for allowed_block in block.allowed_nested_blocks
        ]

    def assert_allowed_nested_blocks(self, block, message_blocks=[]):
        self.assertEqual(
            self.get_allowed_blocks(block),
            self.ALLOWED_NESTED_BLOCKS + message_blocks
        )
        from sys import modules
        xmodule_mock = Mock()
        fake_modules = {
            'xmodule': xmodule_mock,
            'xmodule.video_module': xmodule_mock.video_module,
            'xmodule.video_module.video_module': xmodule_mock.video_module.video_module,
            'imagemodal': Mock(),
            'ooyala_player': Mock(),
        }
        with patch.dict(modules, fake_modules):
            self.assertEqual(
                self.get_allowed_blocks(block),
                self.ALLOWED_NESTED_BLOCKS + self.ADDITIONAL_BLOCKS + message_blocks
            )


def instantiate_block(cls, fields=None):
    """
    Instantiate the given XBlock in a mock runtime.
    """
    fields = fields or {}
    children = fields.pop('children', {})
    field_data = DictFieldData(fields or {})
    block = cls(
        runtime=Mock(),
        field_data=field_data,
        scope_ids=MagicMock()
    )
    block.children = children
    block.runtime.get_block = lambda child_id: children[child_id]
    return block


class DateTimeEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, (datetime, date)):
            return o.isoformat()

        return json.JSONEncoder.default(self, o)



import unittest

from mock import Mock
from xblock.field_data import DictFieldData

from problem_builder.mentoring import MentoringWithExplicitStepsBlock
from problem_builder.step import MentoringStepBlock
from problem_builder.step_review import (ConditionalMessageBlock,
                                         ReviewStepBlock, ScoreSummaryBlock)

from .utils import BlockWithChildrenTestMixin


class TestMentoringBlock(BlockWithChildrenTestMixin, unittest.TestCase):
    def test_student_view_data(self):
        blocks_by_id = {}

        services_mocks = {
            "i18n": Mock(ugettext=lambda string: string)
        }

        mock_runtime = Mock(
            get_block=lambda block_id: blocks_by_id[block_id],
            load_block_type=lambda block: block.__class__,
            service=lambda _, service_id: services_mocks.get(service_id),
            id_reader=Mock(
                get_definition_id=lambda block_id: block_id,
                get_block_type=lambda block_id: blocks_by_id[block_id],
            ),
        )

        def make_block(block_type, data, **kwargs):
            usage_id = str(make_block.id_counter)
            make_block.id_counter += 1
            mock_scope_ids = Mock(usage_id=usage_id)
            block = block_type(
                mock_runtime,
                field_data=DictFieldData(data),
                scope_ids=mock_scope_ids,
                **kwargs
            )
            block.category = 'test'
            blocks_by_id[usage_id] = block
            parent = kwargs.get('for_parent')
            if parent:
                parent.children.append(usage_id)
                block.parent = parent.scope_ids.usage_id
            return block
        make_block.id_counter = 1

        # Create top-level Step Builder block.
        step_builder_data = {
            'display_name': u'My Step Builder',
            'show_title': False,
            'weight': 5.0,
            'max_attempts': 3,
            'extended_feedback': True,
        }
        step_builder = make_block(MentoringWithExplicitStepsBlock, step_builder_data)

        # Create a 'Step' block (as child of 'Step Builder') and add two mock children to it.
        # One of the mocked children implements `student_view_data`, while the other one does not.
        child_a = Mock(spec=['student_view_data'])
        child_a.category = 'test'
        child_a.scope_ids = Mock(usage_id='child_a')
        child_a.student_view_data.return_value = 'child_a_json'
        blocks_by_id['child_a'] = child_a

        child_b = Mock(spec=[])
        child_b.scope_ids = Mock(usage_id='child_b')
        child_b.category = 'test'
        blocks_by_id['child_b'] = child_b

        step_data = {
            'display_name': u'First Step',
            'show_title': True,
            'next_button_label': u'Next Question',
            'message': u'This is the message.',
            'children': [child_a.scope_ids.usage_id, child_b.scope_ids.usage_id],
        }
        make_block(MentoringStepBlock, step_data, for_parent=step_builder)

        # Create a 'Step Review' block (as child of 'Step Builder').
        review_step_data = {
            'display_name': u'My Review Step',
        }
        review_step = make_block(ReviewStepBlock, review_step_data, for_parent=step_builder)

        # Create 'Score Summary' block as child of 'Step Review'.
        make_block(ScoreSummaryBlock, {}, for_parent=review_step)

        # Create 'Conditional Message' block as child of 'Step Review'.
        conditional_message_data = {
            'content': u'This message is conditional',
            'score_condition': u'perfect',
            'num_attempts_condition': u'can_try_again',
        }
        make_block(ConditionalMessageBlock, conditional_message_data, for_parent=review_step)

        expected = {
            'block_id': u'1',
            'display_name': step_builder_data['display_name'],
            'title': step_builder_data['display_name'],
            'show_title': step_builder_data['show_title'],
            'weight': step_builder_data['weight'],
            'max_attempts': step_builder_data['max_attempts'],
            'extended_feedback': step_builder_data['extended_feedback'],
            'components': [
                {
                    'block_id': u'2',
                    'type': 'sb-step',
                    'display_name': step_data['display_name'],
                    'title': step_data['display_name'],
                    'show_title': step_data['show_title'],
                    'next_button_label': step_data['next_button_label'],
                    'message': step_data['message'],
                    'components': ['child_a_json'],
                },
                {
                    'block_id': u'3',
                    'type': 'sb-review-step',
                    'display_name': review_step_data['display_name'],
                    'title': review_step_data['display_name'],
                    'components': [
                        {
                            'block_id': u'4',
                            'display_name': "Score Summary",
                            'type': 'sb-review-score',
                        },
                        {
                            'block_id': u'5',
                            'display_name': "Conditional Message",
                            'type': 'sb-conditional-message',
                            'content': conditional_message_data['content'],
                            'score_condition': conditional_message_data['score_condition'],
                            'num_attempts_condition': conditional_message_data['num_attempts_condition'],
                        },
                    ],
                },
            ],
        }
        self.assertEqual(step_builder.student_view_data(), expected)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#
import json
from functools import wraps
from textwrap import dedent

from mock import Mock, patch
from selenium.common.exceptions import NoSuchElementException

from .base_test import ProblemBuilderBaseTest


class MockSubmissionsAPI(object):
    """
    Mock the submissions API, since it's not available in the test environment.
    """
    def __init__(self):
        self.submissions = {}

    def dict_to_key(self, dict_key):
        return (dict_key['student_id'], dict_key['course_id'], dict_key['item_id'], dict_key['item_type'])

    def create_submission(self, dict_key, submission):
        record = dict(
            student_item=dict_key,
            attempt_number=Mock(),
            submitted_at=Mock(),
            created_at=Mock(),
            answer=submission,
        )
        self.submissions[self.dict_to_key(dict_key)] = record
        return record

    def get_submissions(self, key, limit=1):
        assert limit == 1
        key = self.dict_to_key(key)
        if key in self.submissions:
            return [self.submissions[key]]
        return []


def check_dashboard_and_report(fixture, set_mentoring_values=True, **kwargs):
    """
    Decorator for dashboard test methods.

    - Sets up the given fixture
    - Runs the decorated test
    - Clicks the download link
    - Opens the report
    - Runs the decorated test again against the report

    Any extra keyword arguments are passed to the dashboard XBlock.
    """
    def wrapper(test):
        @wraps(test)
        def wrapped(test_case):
            test_case._install_fixture(fixture)
            if kwargs:
                dashboard = test_case.vertical.get_child('test-scenario.pb-dashboard.d0.u0')
                for key, value in kwargs.items():
                    setattr(dashboard, key, value)
                dashboard.save()
            if set_mentoring_values:
                test_case._set_mentoring_values()
            test_case.go_to_view('student_view')
            test(test_case)
            download_link = test_case.browser.find_element_by_css_selector('.report-download-link')
            download_link.click()
            test_case.browser.get(download_link.get_attribute('href'))
            test_case.assertRegexpMatches(test_case.browser.current_url, r'^data:text/html;base64,')
            test(test_case)
        return wrapped
    return wrapper


class TestDashboardBlock(ProblemBuilderBaseTest):
    """
    Test the Student View of a dashboard XBlock linked to some problem builder blocks
    """
    SIMPLE_DASHBOARD = """<pb-dashboard mentoring_ids='["dummy-value"]'/>"""
    ALTERNATIVE_DASHBOARD = dedent("""
    <pb-dashboard mentoring_ids='["dummy-value"]' show_numbers="false"
        average_labels='{"Step 1": "Avg.", "Step 2":"Mean", "Step 3":"Second Quartile"}'
        header_html='&lt;p id="header-paragraph"&gt;Header&lt;/p&gt;'
        footer_html='&lt;p id="footer-paragraph"&gt;Footer&lt;/p&gt;'
    />
    """)
    HIDE_QUESTIONS_DASHBOARD = dedent("""
    <pb-dashboard mentoring_ids='["dummy-value"]'
        exclude_questions='{"Step 1": [2, 3], "Step 2":[3], "Step 3":[2]}'
    />
    """)
    MALFORMED_HIDE_QUESTIONS_DASHBOARD = dedent("""
    <pb-dashboard mentoring_ids='["dummy-value"]'
      exclude_questions='{"Step 1": "1234", "Step 2":[3], "Step 3":[2]}'
    />
    """)

    # Clean up screenshots if the tests pass
    cleanup_on_success = True

    def setUp(self):
        super(TestDashboardBlock, self).setUp()

        # Apply a whole bunch of patches that are needed in lieu of the LMS/CMS runtime and edx-submissions:

        def get_mentoring_blocks(dashboard_block, mentoring_ids, ignore_errors=True):
            return [dashboard_block.runtime.get_block(key) for key in dashboard_block.get_parent().children[:-1]]

        mock_submisisons_api = MockSubmissionsAPI()
        patches = (
            (
                "problem_builder.dashboard.DashboardBlock._get_submission_key",
                lambda _, child_id: dict(student_id="student", course_id="course", item_id=child_id, item_type="pb-mcq")
            ),
            (
                "problem_builder.sub_api.SubmittingXBlockMixin.student_item_key",
                property(lambda block: dict(
                    student_id="student", course_id="course", item_id=block.scope_ids.usage_id, item_type="pb-mcq"
                ))
            ),
            ("problem_builder.dashboard.DashboardBlock.get_mentoring_blocks", get_mentoring_blocks),
            ("problem_builder.dashboard.sub_api", mock_submisisons_api),
            ("problem_builder.mcq.sub_api", mock_submisisons_api),
            (
                "problem_builder.mentoring.MentoringBlock.url_name",
                property(lambda block: block.display_name)
            )
        )
        for p in patches:
            patcher = patch(*p)
            patcher.start()
            self.addCleanup(patcher.stop)

    def _install_fixture(self, dashboard_xml):
        self.load_scenario("dashboard.xml", {'dashboard': dashboard_xml}, load_immediately=True)
        self.vertical = self.load_root_xblock()

    def _get_cell_contents(self, cell):
        try:
            visible_text = cell.find_element_by_css_selector('span:not(.sr)').text
        except NoSuchElementException:
            visible_text = ""
        screen_reader_text = cell.find_element_by_css_selector('span.sr')
        return visible_text, screen_reader_text.text

    def _assert_cell_contents(self, cell, expected_visible_text, expected_screen_reader_text):
        visible_text, screen_reader_text = self._get_cell_contents(cell)
        self.assertEqual(visible_text, expected_visible_text)
        self.assertEqual(screen_reader_text, expected_screen_reader_text)

    def _format_sr_text(self, visible_text):
        return "Score: {value}".format(value=visible_text)

    def _set_mentoring_values(self):
        pbs = self.browser.find_elements_by_css_selector('.mentoring')
        for pb in pbs:
            mcqs = pb.find_elements_by_css_selector('fieldset.choices')
            for idx, mcq in enumerate(mcqs):
                choices = mcq.find_elements_by_css_selector('.choices .choice label')
                choices[idx].click()
            self.click_submit(pb)

    @check_dashboard_and_report(SIMPLE_DASHBOARD, set_mentoring_values=False)
    def test_empty_dashboard(self):
        """
        Test that when the student has not submitted any question answers, we still see
        the dashboard, and its lists all the MCQ questions in the way we expect.
        """
        dashboard = self.browser.find_element_by_css_selector('.pb-dashboard')
        step_headers = dashboard.find_elements_by_css_selector('thead')
        self.assertEqual(len(step_headers), 3)
        self.assertEqual([hdr.text for hdr in step_headers], ["Step 1", "Step 2", "Step 3"])
        steps = dashboard.find_elements_by_css_selector('tbody')
        self.assertEqual(len(steps), 3)

        for step in steps:
            mcq_rows = step.find_elements_by_css_selector('tr')
            self.assertTrue(2 <= len(mcq_rows) <= 3)
            for mcq in mcq_rows:
                cell = mcq.find_element_by_css_selector('td:last-child')
                self._assert_cell_contents(cell, '', 'No value yet')

    @check_dashboard_and_report(SIMPLE_DASHBOARD)
    def test_dashboard(self):
        """
        Submit an answer to each MCQ, then check that the dashboard reflects those answers.
        """
        dashboard = self.browser.find_element_by_css_selector('.pb-dashboard')
        headers = dashboard.find_elements_by_class_name('report-header')
        self.assertEqual(len(headers), 0)
        footers = dashboard.find_elements_by_class_name('report-footer')
        self.assertEqual(len(footers), 0)
        steps = dashboard.find_elements_by_css_selector('tbody')
        self.assertEqual(len(steps), 3)
        expected_values = ('1', '2', '3', '4', 'B')

        for step_num, step in enumerate(steps):
            mcq_rows = step.find_elements_by_css_selector('tr:not(.avg-row)')
            self.assertTrue(2 <= len(mcq_rows) <= 3)
            for mcq in mcq_rows:
                cell = mcq.find_element_by_css_selector('td.value')
                visible_text, screen_reader_text = self._get_cell_contents(cell)
                self.assertIn(visible_text, expected_values)
                self.assertIn(screen_reader_text, map(self._format_sr_text, expected_values))
            # Check the average:
            avg_row = step.find_element_by_css_selector('tr.avg-row')
            left_col = avg_row.find_element_by_css_selector('.desc')
            self.assertEqual(left_col.text, "Average")
            right_col = avg_row.find_element_by_css_selector('.value')
            expected_average = {0: "2", 1: "3", 2: "1"}[step_num]
            self._assert_cell_contents(right_col, expected_average, self._format_sr_text(expected_average))

    @check_dashboard_and_report(SIMPLE_DASHBOARD, visual_rules=json.dumps({
        'background': '/static/test/swoop-bg.png',
    }))
    def test_dashboard_image(self):
        """
        Test that the dashboard image is displayed correctly, both on the page
        and it the report. We allow minor differences here as the report
        screenshot is not a perfect match.
        """
        self.assertScreenshot('.pb-dashboard-visual svg', 'dashboard-image', threshold=3000)

    @check_dashboard_and_report(SIMPLE_DASHBOARD, visual_rules=json.dumps({
        'background': ('//raw.githubusercontent.com/open-craft/problem-builder/master/'
                       'problem_builder/static/test/swoop-bg.png'),
    }))
    def test_dashboard_image_cross_domain(self):
        """
        Test that cross-domain dashboard images are displayed correctly. We
        allow minor differences here as the report screenshot is not a perfect
        match.
        """
        self.assertScreenshot('.pb-dashboard-visual svg', 'dashboard-image', threshold=3000)

    @check_dashboard_and_report(
        SIMPLE_DASHBOARD,
        visual_rules=json.dumps({
            'background': '/static/test/swoop-bg.png',
            'images': [
                '/static/test/swoop-step1.png',
                '/static/test/swoop-step2.png',
            ],
        }),
        color_rules='\n'.join([
            '0: grey',
            'x <= 2: #aa2626',
            'x <= 3: #e7ce76',
            '#84b077',
        ]),
    )
    def test_dashboard_image_overlay(self):
        """
        Test that image overlays are displayed correctly on the dashboard.
        """
        self.assertScreenshot('.pb-dashboard-visual svg', 'dashboard-image-overlay', threshold=3000)

    @check_dashboard_and_report(ALTERNATIVE_DASHBOARD)
    def test_dashboard_alternative(self):
        """
        Submit an answer to each MCQ, then check that the dashboard reflects those answers with alternative
        configuration:

        * Average label is "Avg." instead of default "Average"
        * Numerical values are not shown
        * Include HTML header and footer snippets
        """
        dashboard = self.browser.find_element_by_css_selector('.pb-dashboard')
        header_p = dashboard.find_element_by_id('header-paragraph')
        self.assertEquals(header_p.text, 'Header')
        footer_p = dashboard.find_element_by_id('footer-paragraph')
        self.assertEquals(footer_p.text, 'Footer')
        steps = dashboard.find_elements_by_css_selector('tbody')
        self.assertEqual(len(steps), 3)

        average_labels = ["Avg.", "Mean", "Second Quartile"]

        expected_values = ('1', '2', '3', '4', 'B')

        for step_num, step in enumerate(steps):
            mcq_rows = step.find_elements_by_css_selector('tr:not(.avg-row)')
            self.assertTrue(2 <= len(mcq_rows) <= 3)
            for mcq in mcq_rows:
                cell = mcq.find_element_by_css_selector('td.value')
                visible_text, screen_reader_text = self._get_cell_contents(cell)
                # this dashboard configured to not show numbers
                self.assertEqual(visible_text, '')
                # but screen reader content still added
                self.assertIn(screen_reader_text, map(self._format_sr_text, expected_values))
            # Check the average:
            avg_row = step.find_element_by_css_selector('tr.avg-row')
            left_col = avg_row.find_element_by_css_selector('.desc')
            self.assertEqual(left_col.text, average_labels[step_num])
            right_col = avg_row.find_element_by_css_selector('.value')
            expected_average = {0: "2", 1: "3", 2: "1"}[step_num]
            self._assert_cell_contents(right_col, '', self._format_sr_text(expected_average))

    @check_dashboard_and_report(HIDE_QUESTIONS_DASHBOARD)
    def test_dashboard_exclude_questions(self):
        """
        Submit an answer to each MCQ, then check that the dashboard ignores questions it is configured to ignore
        """
        dashboard = self.browser.find_element_by_css_selector('.pb-dashboard')
        steps = dashboard.find_elements_by_css_selector('tbody')
        self.assertEqual(len(steps), 3)
        expected_values = ('1', '2', '3', '4')

        lengths = [1, 2, 1]

        for step_num, step in enumerate(steps):
            mcq_rows = step.find_elements_by_css_selector('tr:not(.avg-row)')
            self.assertEqual(len(mcq_rows), lengths[step_num])
            for mcq in mcq_rows:
                cell = mcq.find_element_by_css_selector('td.value')
                visible_text, screen_reader_text = self._get_cell_contents(cell)
                self.assertIn(visible_text, expected_values)
                self.assertIn(screen_reader_text, map(self._format_sr_text, expected_values))
            # Check the average:
            avg_row = step.find_element_by_css_selector('tr.avg-row')
            left_col = avg_row.find_element_by_css_selector('.desc')
            self.assertEqual(left_col.text, "Average")
            right_col = avg_row.find_element_by_css_selector('.value')
            expected_average = {0: "1", 1: "3", 2: "1"}[step_num]
            self._assert_cell_contents(right_col, expected_average, self._format_sr_text(expected_average))

    @check_dashboard_and_report(MALFORMED_HIDE_QUESTIONS_DASHBOARD)
    def test_dashboard_malformed_exclude_questions(self):
        """
        Submit an answer to each MCQ, then check that the dashboard ignores questions it is configured to ignore
        """
        dashboard = self.browser.find_element_by_css_selector('.pb-dashboard')
        steps = dashboard.find_elements_by_css_selector('tbody')
        self.assertEqual(len(steps), 3)

        expected_values = ('1', '2', '3', '4')

        lengths = [3, 2, 1]

        for step_num, step in enumerate(steps):
            mcq_rows = step.find_elements_by_css_selector('tr:not(.avg-row)')
            self.assertEqual(len(mcq_rows), lengths[step_num])
            for mcq in mcq_rows:
                cell = mcq.find_element_by_css_selector('td.value')
                visible_text, screen_reader_text = self._get_cell_contents(cell)
                self.assertIn(visible_text, expected_values)
                self.assertIn(screen_reader_text, map(self._format_sr_text, expected_values))
            # Check the average:
            avg_row = step.find_element_by_css_selector('tr.avg-row')
            left_col = avg_row.find_element_by_css_selector('.desc')
            self.assertEqual(left_col.text, "Average")
            right_col = avg_row.find_element_by_css_selector('.value')
            expected_average = {0: "2", 1: "3", 2: "1"}[step_num]
            self._assert_cell_contents(right_col, expected_average, self._format_sr_text(expected_average))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Answer',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50, db_index=True)),
                ('student_id', models.CharField(max_length=32, db_index=True)),
                ('course_id', models.CharField(max_length=50, db_index=True)),
                ('student_input', models.TextField(default=u'', blank=True)),
                ('created_on', models.DateTimeField(auto_now_add=True, verbose_name=u'created on')),
                ('modified_on', models.DateTimeField(auto_now=True, verbose_name=u'modified on')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='answer',
            unique_together=set([('student_id', 'course_id', 'name')]),
        ),
    ]



"""
Celery task for CSV student answer export.
"""
import time

import six
from django.contrib.auth.models import User
from django.db.models import F

from celery.task import task
from celery.utils.log import get_task_logger
from lms.djangoapps.instructor_task.models import ReportStore
from opaque_keys import InvalidKeyError
from opaque_keys.edx.keys import CourseKey, UsageKey
from problem_builder.answer import AnswerBlock
from xmodule.modulestore.django import modulestore
from xmodule.modulestore.exceptions import ItemNotFoundError

from .mcq import MCQBlock, RatingBlock
from .mrq import MRQBlock
from .questionnaire import QuestionnaireAbstractBlock
from .sub_api import sub_api

logger = get_task_logger(__name__)


@task()
def export_data(course_id, source_block_id_str, block_types, user_ids, match_string):
    """
    Exports student answers to all supported questions to a CSV file.
    """
    start_timestamp = time.time()

    logger.debug("Beginning data export")
    try:
        course_key = CourseKey.from_string(course_id)
        usage_key = UsageKey.from_string(source_block_id_str)
    except InvalidKeyError:
        raise ValueError("Could not find the specified Block ID.")

    src_block = modulestore().get_item(usage_key)
    course_key_str = six.text_type(course_key)
    type_map = {cls.__name__: cls for cls in [MCQBlock, MRQBlock, RatingBlock, AnswerBlock]}

    if not block_types:
        block_types = tuple(type_map.values())
    else:
        block_types = tuple(type_map[class_name] for class_name in block_types)

    # Build an ordered list of blocks to include in the export
    blocks_to_include = []

    def scan_for_blocks(block):
        """ Recursively scan the course tree for blocks of interest """
        if isinstance(block, block_types):
            blocks_to_include.append(block)
        elif block.has_children:
            for child_id in block.children:
                try:
                    scan_for_blocks(block.runtime.get_block(child_id))
                except ItemNotFoundError:
                    # Blocks may refer to missing children. Don't break in this case.
                    pass

    scan_for_blocks(src_block)

    # Define the header row of our CSV:
    rows = []
    rows.append(
        ["Section", "Subsection", "Unit", "Type", "Question", "Answer", "Username", "User ID", "User E-mail"]
    )

    # Collect results for each block in blocks_to_include
    for block in blocks_to_include:
        if not user_ids:
            results = _extract_data(course_key_str, block, None, match_string)
            rows += results
        else:
            for user_id in user_ids:
                results = _extract_data(course_key_str, block, user_id, match_string)
                rows += results

    # Generate the CSV:
    filename = u"pb-data-export-{}.csv".format(time.strftime("%Y-%m-%d-%H%M%S", time.gmtime(start_timestamp)))
    report_store = ReportStore.from_config(config_name='GRADES_DOWNLOAD')
    report_store.store_rows(course_key, filename, rows)

    generation_time_s = time.time() - start_timestamp
    logger.debug("Done data export - took {} seconds".format(generation_time_s))

    return {
        "error": None,
        "report_filename": filename,
        "start_timestamp": start_timestamp,
        "generation_time_s": generation_time_s,
        "display_data": [] if len(rows) == 1 else rows[1:1001]  # Limit to preview of 1000 items
    }


def _extract_data(course_key_str, block, user_id, match_string):
    """
    Extract results for `block`.
    """
    rows = []

    # Extract info for "Section", "Subsection", and "Unit" columns
    section_name, subsection_name, unit_name = _get_context(block)

    # Extract info for "Type" column
    block_type = _get_type(block)

    # Extract info for "Question" column
    block_question = _get_question(block)

    # Extract info for "Answer" and "Username" columns
    # - Get all of the most recent student submissions for this block:
    submissions = tuple(_get_submissions(course_key_str, block, user_id))

    student_ids = [submission['student_id'] for submission in submissions]
    users = get_users_by_anonymous_ids(student_ids)

    # - For each submission, look up student's username, email and answer:
    answer_cache = {}
    for submission in submissions:
        username, _user_id, user_email = users.get(
            submission['student_id'],
            (submission['student_id'], 'N/A', 'N/A')
        )
        answer = _get_answer(block, submission, answer_cache)

        # Short-circuit if answer does not match search criteria
        if not match_string.lower() in answer.lower():
            continue

        rows.append([
            section_name,
            subsection_name,
            unit_name,
            block_type,
            block_question,
            answer,
            username,
            _user_id,
            user_email
        ])

    return rows


def _get_context(block):
    """
    Return section, subsection, and unit names for `block`.
    """
    block_names_by_type = {}
    block_iter = block
    while block_iter:
        block_iter_type = block_iter.scope_ids.block_type
        block_names_by_type[block_iter_type] = block_iter.display_name_with_default
        block_iter = block_iter.get_parent() if block_iter.parent else None
    section_name = block_names_by_type.get('chapter', '')
    subsection_name = block_names_by_type.get('sequential', '')
    unit_name = block_names_by_type.get('vertical', '')
    return section_name, subsection_name, unit_name


def _get_type(block):
    """
    Return type of `block`.
    """
    return block.scope_ids.block_type


def _get_question(block):
    """
    Return question for `block`; default to question ID if `question` is not set.
    """
    return block.question or block.name


def _get_submissions(course_key_str, block, user_id):
    """
    Return submissions for `block`.
    """
    # Load the actual student submissions for `block`.
    # Note this requires one giant query that retrieves all student submissions for `block` at once.
    block_id = six.text_type(block.scope_ids.usage_id.replace(branch=None, version_guid=None))
    block_type = _get_type(block)
    if block_type == 'pb-answer':
        block_id = block.name  # item_id of Long Answer submission matches question ID and not block ID
    if not user_id:
        return sub_api.get_all_submissions(course_key_str, block_id, block_type)
    else:
        student_dict = {
            'student_id': user_id,
            'item_id': block_id,
            'course_id': course_key_str,
            'item_type': block_type,
        }
        return sub_api.get_submissions(student_dict, limit=1)


def get_users_by_anonymous_ids(anonymous_ids):
    """
    Return users by anonymous_ids using AnonymousUserId lookup table.
    """
    if not anonymous_ids:
        return None

    users = User.objects.filter(
        anonymoususerid__anonymous_user_id__in=anonymous_ids
    ).annotate(
        anonymous_user_id=F('anonymoususerid__anonymous_user_id')
    ).values(
        'anonymous_user_id', 'username', 'id', 'email'
    ).iterator()

    return {
        user['anonymous_user_id']: (user['username'], user['id'], user['email']) for user in users
    }


def _get_answer(block, submission, answer_cache):
    """
    Return answer associated with `submission` to `block`.

    `answer_cache` is a dict that is reset for each block.
    """
    answer = submission['answer']
    if isinstance(block, QuestionnaireAbstractBlock):
        # Convert from answer ID to answer label
        if answer not in answer_cache:
            answer_cache[answer] = block.get_submission_display(answer)
        return answer_cache[answer]
    return answer



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def copy_course_id_to_course_key(apps, schema_editor):
    """
    Iterates over all Answer records for which course_key is not set
    and copies the value of the course_id column into course_key.
    """
    Answer = apps.get_model('problem_builder', 'Answer')
    for answer in Answer.objects.filter(course_key__isnull=True).iterator():
        answer.course_key = answer.course_id
        answer.save()


class Migration(migrations.Migration):

    dependencies = [
        ('problem_builder', '0003_auto_20161124_0755'),
    ]

    operations = [
        migrations.RunPython(code=copy_course_id_to_course_key, reverse_code=migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

import logging

import pkg_resources
import six
from django import utils
from lazy.lazy import lazy
from xblock.core import XBlock
from xblock.fields import List, Scope, String
from xblock.fragment import Fragment
from xblockutils.resources import ResourceLoader
from xblockutils.studio_editable import (NestedXBlockSpec,
                                         StudioContainerWithNestedXBlocksMixin,
                                         StudioEditableXBlockMixin,
                                         XBlockWithPreviewMixin)

from problem_builder.answer import AnswerBlock, AnswerRecapBlock
from problem_builder.completion import CompletionBlock
from problem_builder.mcq import MCQBlock, RatingBlock
from problem_builder.mixins import (
    EnumerableChildMixin, StepParentMixin, StudentViewUserStateMixin,
    StudentViewUserStateResultsTransformerMixin)
from problem_builder.mrq import MRQBlock
from problem_builder.plot import PlotBlock
from problem_builder.slider import SliderBlock
from problem_builder.table import MentoringTableBlock

from .utils import I18NService

log = logging.getLogger(__name__)
loader = ResourceLoader(__name__)


# Make '_' a no-op so we can scrape strings
def _(text):
    return text


def _normalize_id(key):
    """
    Helper method to normalize a key to avoid issues where some keys have version/branch and others don't.
    e.g. self.scope_ids.usage_id != self.runtime.get_block(self.scope_ids.usage_id).scope_ids.usage_id
    """
    if hasattr(key, "for_branch"):
        key = key.for_branch(None)
    if hasattr(key, "for_version"):
        key = key.for_version(None)
    return key


class Correctness(object):
    CORRECT = 'correct'
    PARTIAL = 'partial'
    INCORRECT = 'incorrect'


@XBlock.needs('i18n')
class MentoringStepBlock(
    StudioEditableXBlockMixin, StudioContainerWithNestedXBlocksMixin, XBlockWithPreviewMixin,
    EnumerableChildMixin, StepParentMixin, StudentViewUserStateResultsTransformerMixin,
    StudentViewUserStateMixin, XBlock, I18NService
):
    """
    An XBlock for a step.
    """
    CAPTION = _(u"Step")
    STUDIO_LABEL = _(u"Mentoring Step")
    CATEGORY = 'sb-step'
    USER_STATE_FIELDS = ['student_results']

    # Settings
    display_name = String(
        display_name=_("Step Title"),
        help=_('Leave blank to use sequential numbering'),
        default="",
        scope=Scope.content
    )

    # User state
    student_results = List(
        # Store results of student choices.
        default=[],
        scope=Scope.user_state
    )

    next_button_label = String(
        display_name=_("Next Button Label"),
        help=_("Customize the text of the 'Next' button."),
        default=_("Next Step")
    )

    message = String(
        display_name=_("Message"),
        help=_("Feedback or instructional message which pops up after submitting."),
    )

    editable_fields = ('display_name', 'show_title', 'next_button_label', 'message')

    @lazy
    def siblings(self):
        return self.get_parent().step_ids

    @property
    def is_last_step(self):
        parent = self.get_parent()
        return self.step_number == len(parent.step_ids)

    @property
    def allowed_nested_blocks(self):
        """
        Returns a list of allowed nested XBlocks. Each item can be either
        * An XBlock class
        * A NestedXBlockSpec

        If XBlock class is used it is assumed that this XBlock is enabled and allows multiple instances.
        NestedXBlockSpec allows explicitly setting disabled/enabled state, disabled reason (if any) and single/multiple
        instances
        """
        additional_blocks = []
        try:
            from xmodule.video_module.video_module import VideoDescriptor
            additional_blocks.append(NestedXBlockSpec(
                VideoDescriptor, category='video', label=_(u"Video")
            ))
        except ImportError:
            pass
        try:
            from imagemodal import ImageModal
            additional_blocks.append(NestedXBlockSpec(
                ImageModal, category='imagemodal', label=_(u"Image Modal")
            ))
        except ImportError:
            pass

        try:
            from ooyala_player import OoyalaPlayerBlock
            additional_blocks.append(NestedXBlockSpec(
                OoyalaPlayerBlock, category='ooyala-player', label=_(u"Ooyala Player")
            ))
        except ImportError:
            pass

        return [
            NestedXBlockSpec(AnswerBlock, boilerplate='studio_default'),
            MCQBlock, RatingBlock, MRQBlock, CompletionBlock,
            NestedXBlockSpec(None, category="html", label=self._("HTML")),
            AnswerRecapBlock, MentoringTableBlock, PlotBlock, SliderBlock
        ] + additional_blocks

    @property
    def has_question(self):
        return any(getattr(child, 'answerable', False) for child in self.steps)

    def submit(self, submissions):
        """ Handle a student submission. This is called by the parent XBlock. """
        log.info(u'Received submissions: {}'.format(submissions))

        # Submit child blocks (questions) and gather results
        submit_results = []
        for child in self.steps:
            if child.name and child.name in submissions:
                submission = submissions[child.name]
                child_result = child.submit(submission)
                submit_results.append([child.name, child_result])
                child.save()

        # Update results stored for this step
        self.reset()
        for result in submit_results:
            self.student_results.append(result)
        self.save()

        return {
            'message': 'Success!',
            'step_status': self.answer_status,
            'results': submit_results,
        }

    @XBlock.json_handler
    def get_results(self, queries, suffix=''):
        results = {}
        answers = dict(self.student_results)
        for question in self.steps:
            previous_results = answers[question.name]
            result = question.get_results(previous_results)
            results[question.name] = result

        # Add 'message' to results? Looks like it's not used on the client ...
        return {
            'results': results,
            'step_status': self.answer_status,
        }

    def reset(self):
        while self.student_results:
            self.student_results.pop()

    @property
    def answer_status(self):
        if all(result[1]['status'] == 'correct' for result in self.student_results):
            answer_status = Correctness.CORRECT
        elif all(result[1]['status'] == 'incorrect' for result in self.student_results):
            answer_status = Correctness.INCORRECT
        else:
            answer_status = Correctness.PARTIAL
        return answer_status

    def author_edit_view(self, context):
        """
        Add some HTML to the author view that allows authors to add child blocks.
        """
        local_context = dict(context)
        local_context['author_edit_view'] = True
        fragment = super(MentoringStepBlock, self).author_edit_view(local_context)
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder.css'))
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder-edit.css'))
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder-tinymce-content.css'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/util.js'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/container_edit.js'))
        fragment.initialize_js('ProblemBuilderContainerEdit')
        return fragment

    def mentoring_view(self, context=None):
        """ Mentoring View """
        return self._render_view(context, 'mentoring_view')

    @staticmethod
    def resource_string(path):
        """Handy helper for getting resources from our kit."""
        data = pkg_resources.resource_string(__name__, path)
        return data.decode("utf8")

    def get_translation_content(self):
        try:
            # here we need to split the lang code and need to change - to _ and post - characters to
            # upper case since we have local directories like ja_JP, etc instead of ja-jp, etc
            language = utils.translation.get_language().split('-')
            if len(language) == 2:
                new_lang = language[0] + "_" + language[1].upper()
            else:
                new_lang = utils.translation.get_language()
            return self.resource_string('public/js/translations/{lang}/textjs.js'.format(
                lang=new_lang
            ))
        except IOError:
            return self.resource_string('public/js/translations/en/textjs.js')

    def _render_view(self, context, view):
        """ Actually renders a view """
        rendering_for_studio = False
        if context:  # Workbench does not provide context
            rendering_for_studio = context.get('author_preview_view')

        fragment = Fragment()
        child_contents = []

        for child_id in self.children:
            child = self.runtime.get_block(child_id)
            if child is None:  # child should not be None but it can happen due to bugs or permission issues
                child_contents.append(u"<p>[{}]</p>".format(self._(u"Error: Unable to load child component.")))
            else:
                if rendering_for_studio and isinstance(child, PlotBlock):
                    # Don't use view to render plot blocks in Studio.
                    # This is necessary because:
                    # - student_view of plot block uses submissions API to retrieve results,
                    #   which causes "SubmissionRequestError" in Studio.
                    # - author_preview_view does not supply JS code for plot that JS code for step depends on
                    #   (step calls "update" on plot to get latest data during rendering).
                    child_contents.append(u"<p>{}</p>".format(child.display_name))
                else:
                    child_fragment = self._render_child_fragment(child, context, view)
                    fragment.add_frag_resources(child_fragment)
                    child_contents.append(child_fragment.content)

        fragment.add_content(loader.render_django_template('templates/html/step.html', {
            'self': self,
            'title': self.display_name,
            'show_title': self.show_title,
            'child_contents': child_contents,
        }, i18n_service=self.i18n_service))

        fragment.add_javascript(self.get_translation_content())
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/step.js'))
        fragment.initialize_js('MentoringStepBlock')

        return fragment

    def student_view_data(self, context=None):
        """
        Returns a JSON representation of the student_view of this XBlock,
        retrievable from the Course XBlock API.
        """
        components = []

        for child_id in self.children:
            child = self.runtime.get_block(child_id)
            if hasattr(child, 'student_view_data'):
                components.append(child.student_view_data(context))

        return {
            'block_id': six.text_type(self.scope_ids.usage_id),
            'display_name': self.display_name_with_default,
            'type': self.CATEGORY,
            'title': self.display_name_with_default,
            'show_title': self.show_title,
            'next_button_label': self.next_button_label,
            'message': self.message,
            'components': components,
        }



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Answer'
        db.create_table('problem_builder_answer', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=50, db_index=True)),
            ('student_id', self.gf('django.db.models.fields.CharField')(max_length=32, db_index=True)),
            ('course_id', self.gf('django.db.models.fields.CharField')(max_length=50, db_index=True)),
            ('student_input', self.gf('django.db.models.fields.TextField')(default='', blank=True)),
            ('created_on', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('modified_on', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal('problem_builder', ['Answer'])

        # Adding unique constraint on 'Answer', fields ['student_id', 'course_id', 'name']
        db.create_unique('problem_builder_answer', ['student_id', 'course_id', 'name'])

    def backwards(self, orm):
        # Removing unique constraint on 'Answer', fields ['student_id', 'course_id', 'name']
        db.delete_unique('problem_builder_answer', ['student_id', 'course_id', 'name'])

        # Deleting model 'Answer'
        db.delete_table('problem_builder_answer')

    models = {
        'problem_builder.answer': {
            'Meta': {'unique_together': "(('student_id', 'course_id', 'name'),)", 'object_name': 'Answer'},
            'course_id': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'created_on': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified_on': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'student_id': ('django.db.models.fields.CharField', [], {'max_length': '32', 'db_index': 'True'}),
            'student_input': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'})
        }
    }

    complete_apps = ['problem_builder']



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

# Imports ###########################################################

import ddt

from .base_test import MentoringBaseTest

# Classes ###########################################################


@ddt.ddt
class QuestionnaireBlockTest(MentoringBaseTest):
    def _get_choice_label_text(self, choice):
        return choice.find_element_by_css_selector('label').text

    def _get_inputs(self, choices):
        return [choice.find_element_by_css_selector('input') for choice in choices]

    def assert_messages_empty(self, messages):
        self.assertEqual(messages.text, '')
        self.assertFalse(messages.find_elements_by_xpath('./*'))

    def _click_result_icon(self, choice):
        choice.find_element_by_css_selector(".choice-result").click()

    def test_mcq_choices_rating(self):
        """
        Mentoring MCQ should display tips according to user choice
        """
        # Initial MCQ status
        mentoring = self.go_to_page('Mcq 1')
        mcq1 = mentoring.find_element_by_css_selector('fieldset.choices')
        mcq1_heading = mentoring.find_element_by_id('heading_' + mcq1.get_attribute('id'))
        mcq2 = mentoring.find_element_by_css_selector('fieldset.rating')
        mcq2_heading = mentoring.find_element_by_id('heading_' + mcq2.get_attribute('id'))
        messages = mentoring.find_element_by_css_selector('.messages')
        submit = mentoring.find_element_by_css_selector('.submit input.input-main')

        self.assert_messages_empty(messages)
        self.assertFalse(submit.is_enabled())

        self.assertEqual(mcq1_heading.text, 'Question 1')
        self.assertEqual(mcq1.find_element_by_css_selector('legend').text, 'Do you like this MCQ?')
        self.assertEqual(mcq2_heading.text, 'Question 2')
        self.assertEqual(mcq2.find_element_by_css_selector('legend').text, 'How do you rate this MCQ?')

        mcq1_choices = mcq1.find_elements_by_css_selector('.choices .choice')
        mcq2_choices = mcq2.find_elements_by_css_selector('.rating .choice')

        mcq1_choices_input = self._get_inputs(mcq1_choices)
        mcq2_choices_input = self._get_inputs(mcq2_choices)

        self.assertListEqual(
            [self._get_choice_label_text(choice) for choice in mcq1_choices],
            ["Yes", "Maybe not", "I don't understand"]
        )
        self.assertListEqual(
            [self._get_choice_label_text(choice) for choice in mcq2_choices],
            ['1 - Not good at all', '2', '3', '4', '5 - Extremely good', "I don't want to rate it"]
        )

        self.assertListEqual(
            [choice_input.get_attribute('value') for choice_input in mcq1_choices_input],
            ['yes', 'maybenot', 'understand']
        )

        self.assertListEqual(
            [choice_input.get_attribute('value') for choice_input in mcq2_choices_input],
            ['1', '2', '3', '4', '5', 'notwant']
        )

        def submit_answer_and_assert_messages(
                mcq1_answer, mcq2_answer, item_feedback1, item_feedback2,
                feedback1_selector=".choice-tips .tip p",
                feedback2_selector=".choice-tips .tip p"):

            mcq1_choices_input[mcq1_answer].click()
            mcq2_choices_input[mcq2_answer].click()
            self.assertTrue(submit.is_enabled())
            submit.click()
            self.wait_until_disabled(submit)

            mcq1_feedback = mcq1.find_element_by_css_selector(feedback1_selector)
            mcq2_feedback = mcq2.find_element_by_css_selector(feedback2_selector)

            self.assertEqual(mcq1_feedback.text, item_feedback1)
            self.assertTrue(mcq1_feedback.is_displayed())

            self.assertEqual(mcq2_feedback.text, item_feedback2)
            self.assertTrue(mcq2_feedback.is_displayed())

        # Submit button disabled without selecting anything
        self.assertFalse(submit.is_enabled())

        # Submit button stays disabled when there are unfinished mcqs
        mcq1_choices_input[1].click()
        self.assertFalse(submit.is_enabled())

        # Should not show full completion message when wrong answers are selected
        submit_answer_and_assert_messages(0, 2, 'Great!', 'Will do better next time...')
        self.assertEqual(messages.text, '')
        self.assertFalse(messages.is_displayed())

        # When selected answers have no tips display generic feedback message
        submit_answer_and_assert_messages(
            1, 5, 'Feedback message 1', 'Feedback message 2',
            ".feedback .message-content", ".feedback .message-content"
        )
        self.assertEqual(messages.text, '')
        self.assertFalse(messages.is_displayed())

        # Should show full completion when the right answers are selected
        submit_answer_and_assert_messages(0, 3, 'Great!', 'I love good grades.')
        self.assertIn('All is good now...\nCongratulations!', messages.text)
        self.assertTrue(messages.is_displayed())

        # The choice tip containers should have the with-tips class.
        mcq1_tip_containers = mcq1.find_elements_by_css_selector('.choice-tips-container.with-tips')
        mcq2_tip_containers = mcq2.find_elements_by_css_selector('.choice-tips-container.with-tips')
        self.assertEqual(len(mcq1_tip_containers), 3)
        self.assertEqual(len(mcq2_tip_containers), 6)

        # Clicking outside the tips should hide the tips and clear the with-tips class.
        mcq1_tips = mcq1.find_element_by_css_selector(".choice-tips .tip p")
        mcq2_tips = mcq2.find_element_by_css_selector(".choice-tips .tip p")
        mcq1_heading.click()
        mcq2_heading.click()
        mcq1_tip_containers = mcq1.find_elements_by_css_selector('.choice-tips-container.with-tips')
        mcq2_tip_containers = mcq2.find_elements_by_css_selector('.choice-tips-container.with-tips')
        self.assertEqual(len(mcq1_tip_containers), 0)
        self.assertEqual(len(mcq2_tip_containers), 0)
        self.assertFalse(mcq1_tips.is_displayed())
        self.assertFalse(mcq2_tips.is_displayed())

    def test_mrq_with_comments(self):
        mentoring = self.go_to_page('Mrq With Comments 1')
        mcq = mentoring.find_element_by_css_selector('fieldset.choices')
        messages = mentoring.find_element_by_css_selector('.messages')
        submit = mentoring.find_element_by_css_selector('.submit input.input-main')

        self.assertEqual(messages.text, '')
        self.assertFalse(messages.find_elements_by_xpath('./*'))
        self.assertFalse(submit.is_enabled())

        mcq_legend = mcq.find_element_by_css_selector('legend')
        self.assertEqual(mcq_legend.text, 'What do you like in this MRQ?')

        mcq_heading = mentoring.find_element_by_id('heading_' + mcq.get_attribute('id'))
        self.assertEqual(mcq_heading.text, 'Question')

        mcq_choices = mcq.find_elements_by_css_selector('.choices .choice')

        self.assertEqual(len(mcq_choices), 4)
        self.assertListEqual(
            [self._get_choice_label_text(choice) for choice in mcq_choices],
            ['Its elegance', 'Its beauty', "Its gracefulness", "Its bugs"]
        )

        mcq_choices_input = self._get_inputs(mcq_choices)
        self.assertEqual(mcq_choices_input[0].get_attribute('value'), 'elegance')
        self.assertEqual(mcq_choices_input[1].get_attribute('value'), 'beauty')
        self.assertEqual(mcq_choices_input[2].get_attribute('value'), 'gracefulness')
        self.assertEqual(mcq_choices_input[3].get_attribute('value'), 'bugs')

    def test_mrq_feedback_popups(self):
        mentoring = self.go_to_page('Mrq With Comments 1')

        item_feedbacks = [
            "This is something everyone has to like about this MRQ",
            "This is something everyone has to like about beauty",
            "This MRQ is indeed very graceful",
            "Nah, there aren\\'t any!"
        ]
        self.popup_check(mentoring, item_feedbacks, prefix='div[data-name="mrq_1_1_7"]')

        for index, _ in enumerate(item_feedbacks):
            choice_wrapper = mentoring.find_elements_by_css_selector('div[data-name="mrq_1_1_7"]' + " .choice")[index]
            choice_wrapper.find_element_by_css_selector(".choice-selector input").click()
            item_feedback_icon = choice_wrapper.find_element_by_css_selector(".choice-result")
            if item_feedback_icon.is_displayed():
                item_feedback_icon.click()  # clicking on item feedback icon
            item_feedback_popup = choice_wrapper.find_element_by_css_selector(".choice-tips")

            self.assertFalse(item_feedback_popup.is_displayed())
            self.assertEqual(item_feedback_popup.text, "")

    def _get_questionnaire_options(self, questionnaire):
        result = []
        # this could be a list comprehension, but a bit complicated one - hence explicit loop
        for choice_wrapper in questionnaire.find_elements_by_css_selector(".choice"):
            choice_label = choice_wrapper.find_element_by_css_selector("label")
            result.append(choice_label)
        return result

    @ddt.data(
        'Mrq With Html Choices',
        'Mcq With Html Choices'
    )
    def test_questionnaire_html_choices(self, page):

        mentoring = self.go_to_page(page)

        question = mentoring.find_element_by_css_selector('fieldset legend')
        self.assertIn(
            'What do <strong>you</strong> like in this ',
            question.get_attribute('innerHTML').strip()
        )

        choices_list = mentoring.find_element_by_css_selector(".choices-list")
        messages = mentoring.find_element_by_css_selector('.messages')

        expected_options = [
            "<strong>Its elegance</strong>",
            "<em>Its beauty</em>",
            "<strong>Its gracefulness</strong>",
            '<span style="font-color:red">Its bugs</span>'
        ]

        # Ensure each questionnaire label contains the input item, and the expected option.
        labels = self._get_questionnaire_options(choices_list)
        self.assertEquals(len(labels), len(expected_options))
        for idx, label in enumerate(labels):
            self.assertEquals(len(label.find_elements_by_tag_name('input')), 1)
            self.assertIn(expected_options[idx], label.get_attribute('innerHTML').strip())

        self.assert_messages_empty(messages)

        submit = mentoring.find_element_by_css_selector('.submit input.input-main')
        self.assertFalse(submit.is_enabled())

        inputs = choices_list.find_elements_by_css_selector('.choice-selector input')
        inputs[0].click()
        inputs[1].click()
        inputs[2].click()

        self.assertTrue(submit.is_enabled())
        submit.click()
        self.wait_until_disabled(submit)

        self.assertIn('Congratulations!', messages.text)

    def _get_inner_height(self, elem):
        return elem.size['height'] - \
            int(elem.value_of_css_property("padding-top").replace(u'px', u'')) - \
            int(elem.value_of_css_property("padding-bottom").replace(u'px', u''))

    @ddt.unpack
    @ddt.data(
        ('yes', 33),
        ('maybenot', 60),
        ('understand', 600)
    )
    def test_tip_height(self, choice_value, expected_height):
        mentoring = self.go_to_page("Mcq With Fixed Height Tips")
        choices_list = mentoring.find_element_by_css_selector(".choices-list")
        submit = mentoring.find_element_by_css_selector('.submit input.input-main')

        choice_input_css_selector = ".choice input[value={}]".format(choice_value)
        choice_input = choices_list.find_element_by_css_selector(choice_input_css_selector)
        choice_wrapper = choice_input.find_element_by_xpath("./ancestor::div[@class='choice']")

        choice_input.click()
        self.wait_until_clickable(submit)
        submit.click()
        self.wait_until_disabled(submit)

        self._click_result_icon(choice_wrapper)
        item_feedback_popup = choice_wrapper.find_element_by_css_selector(".choice-tips")
        self.assertTrue(item_feedback_popup.is_displayed())
        feedback_height = self._get_inner_height(item_feedback_popup)
        self.assertEqual(feedback_height, expected_height)


class QuestionnaireBlockAprosThemeTest(QuestionnaireBlockTest):
    """
    Test MRQ/MCQ questions without the LMS theme which is on by default.
    """
    pass



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

# Imports ###########################################################

import logging

import six
from xblock.fields import List, Scope, String
from xblock.fragment import Fragment
from xblock.validation import ValidationMessage
from xblockutils.resources import ResourceLoader

from problem_builder.mixins import (ExpandStaticURLMixin,
                                    StudentViewUserStateMixin)

from .questionnaire import QuestionnaireAbstractBlock
from .sub_api import SubmittingXBlockMixin, sub_api

# Globals ###########################################################

log = logging.getLogger(__name__)
loader = ResourceLoader(__name__)


# Make '_' a no-op so we can scrape strings
def _(text):
    return text

# Classes ###########################################################


class MCQBlock(SubmittingXBlockMixin, StudentViewUserStateMixin, QuestionnaireAbstractBlock, ExpandStaticURLMixin):
    """
    An XBlock used to ask multiple-choice questions
    """
    CATEGORY = 'pb-mcq'
    STUDIO_LABEL = _(u"Multiple Choice Question")
    USER_STATE_FIELDS = ['num_attempts', 'student_choice']

    message = String(
        display_name=_("Message"),
        help=_(
            "General feedback provided when submitting. "
            "(This is not shown if there is a more specific feedback tip for the choice selected by the learner.)"
        ),
        scope=Scope.content,
        default=""
    )

    student_choice = String(
        # {Last input submitted by the student
        default="",
        scope=Scope.user_state,
    )

    correct_choices = List(
        display_name=_("Correct Choice[s]"),
        help=_("Specify the value[s] that students may select for this question to be considered correct."),
        scope=Scope.content,
        list_values_provider=QuestionnaireAbstractBlock.choice_values_provider,
        list_style='set',  # Underered, unique items. Affects the UI editor.
    )
    editable_fields = QuestionnaireAbstractBlock.editable_fields + ('message', 'correct_choices',)

    def describe_choice_correctness(self, choice_value):
        if choice_value in self.correct_choices:
            if len(self.correct_choices) == 1:
                # Translators: This is an adjective, describing a choice as correct
                return self._(u"Correct")
            return self._(u"Acceptable")
        else:
            if len(self.correct_choices) == 1:
                return self._(u"Wrong")
            return self._(u"Not Acceptable")

    def calculate_results(self, submission):
        correct = submission in self.correct_choices
        tips_html = []
        for tip in self.get_tips():
            if submission in tip.values:
                tips_html.append(tip.render('mentoring_view').content)

        formatted_tips = None

        if tips_html:
            formatted_tips = loader.render_template('templates/html/tip_choice_group.html', {
                'tips_html': tips_html,
            })

        self.student_choice = submission

        if sub_api:
            # Also send to the submissions API:
            sub_api.create_submission(self.student_item_key, submission)

        return {
            'submission': submission,
            'message': self.message_formatted,
            'status': 'correct' if correct else 'incorrect',
            'tips': formatted_tips,
            'weight': self.weight,
            'score': 1 if correct else 0,
        }

    def get_results(self, previous_result):
        return self.calculate_results(previous_result['submission'])

    def get_last_result(self):
        return self.get_results({'submission': self.student_choice}) if self.student_choice else {}

    def submit(self, submission):
        log.debug(u'Received MCQ submission: "%s"', submission)
        result = self.calculate_results(submission['value'])
        self.student_choice = submission['value']
        log.debug(u'MCQ submission result: %s', result)
        return result

    def get_author_edit_view_fragment(self, context):
        """
        The options for the 1-5 values of the Likert scale aren't child blocks but we want to
        show them in the author edit view, for clarity.
        """
        fragment = Fragment(u"<p>{}</p>".format(self.question))
        self.render_children(context, fragment, can_reorder=True, can_add=False)
        return fragment

    def validate_field_data(self, validation, data):
        """
        Validate this block's field data.
        """
        super(MCQBlock, self).validate_field_data(validation, data)

        def add_error(msg):
            validation.add(ValidationMessage(ValidationMessage.ERROR, msg))

        def choice_name(choice_value):
            for choice in self.human_readable_choices:
                if choice["value"] == choice_value:
                    return choice["display_name"]
            return choice_value

        all_values = set(self.all_choice_values)
        correct = set(data.correct_choices)

        if all_values and not correct:
            add_error(
                self._(u"You must indicate the correct answer[s], or the student will always get this question wrong.")
            )
        if len(correct) < len(data.correct_choices):
            add_error(self._(u"Duplicate correct choices set"))
        for val in (correct - all_values):
            add_error(
                self._(u"A choice value listed as correct does not exist: {choice}").format(choice=choice_name(val))
            )

    def student_view_data(self, context=None):
        """
        Returns a JSON representation of the student_view of this XBlock,
        retrievable from the Course Block API.
        """
        return {
            'id': self.name,
            'block_id': six.text_type(self.scope_ids.usage_id),
            'display_name': self.display_name_with_default,
            'type': self.CATEGORY,
            'question': self.expand_static_url(self.question),
            'message': self.message,
            'choices': [
                {'value': choice['value'], 'content': self.expand_static_url(choice['display_name'])}
                for choice in self.human_readable_choices
            ],
            'weight': self.weight,
            'tips': [tip.student_view_data() for tip in self.get_tips()],
        }


class RatingBlock(MCQBlock):
    """
    An XBlock used to rate something on a five-point scale, e.g. Likert Scale
    """
    CATEGORY = 'pb-rating'
    STUDIO_LABEL = _(u"Rating Question")

    low = String(
        display_name=_("Low"),
        help=_("Label for low ratings"),
        scope=Scope.content,
        default=_("Less"),
    )
    high = String(
        display_name=_("High"),
        help=_("Label for high ratings"),
        scope=Scope.content,
        default=_("More"),
    )
    FIXED_VALUES = ["1", "2", "3", "4", "5"]
    correct_choices = List(
        display_name=_("Accepted Choice[s]"),
        help=_("Specify the rating value[s] that students may select for this question to be considered correct."),
        scope=Scope.content,
        default=FIXED_VALUES,
        list_values_provider=QuestionnaireAbstractBlock.choice_values_provider,
        list_style='set',  # Underered, unique items. Affects the UI editor.
    )
    editable_fields = MCQBlock.editable_fields + ('low', 'high')

    @property
    def all_choice_values(self):
        return self.FIXED_VALUES + [c.value for c in self.custom_choices]

    @property
    def human_readable_choices(self):
        display_names = ["1 - {}".format(self.low), "2", "3", "4", "5 - {}".format(self.high)]
        return [
            {"display_name": dn, "value": val} for val, dn in zip(self.FIXED_VALUES, display_names)
        ] + super(RatingBlock, self).human_readable_choices

    def get_author_edit_view_fragment(self, context):
        """
        The options for the 1-5 values of the Likert scale aren't child blocks but we want to
        show them in the author edit view, for clarity.
        """
        fragment = Fragment()
        fragment.add_content(loader.render_template('templates/html/ratingblock_edit_preview.html', {
            'question': self.question,
            'low': self.low,
            'high': self.high,
            'accepted_statuses': [None] + [self.describe_choice_correctness(c) for c in "12345"],
        }))
        self.render_children(context, fragment, can_reorder=True, can_add=False)
        return fragment

    @property
    def url_name(self):
        """
        Get the url_name for this block. In Studio/LMS it is provided by a mixin, so we just
        defer to super(). In the workbench or any other platform, we use the name.
        """
        try:
            return super(RatingBlock, self).url_name
        except AttributeError:
            return self.name

    def student_view(self, context):
        fragment = super(RatingBlock, self).student_view(context)
        rendering_for_studio = None
        if context:  # Workbench does not provide context
            rendering_for_studio = context.get('author_edit_view')
        if rendering_for_studio:
            fragment.add_content(loader.render_template('templates/html/rating_edit_footer.html', {
                "url_name": self.url_name
            }))
        return fragment



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

# Imports ###########################################################

import six
from django.utils.html import strip_tags
from lxml import etree
from xblock.core import XBlock
from xblock.fields import List, Scope, String
from xblock.fragment import Fragment
from xblock.validation import ValidationMessage
from xblockutils.resources import ResourceLoader
from xblockutils.studio_editable import StudioEditableXBlockMixin

from problem_builder.mixins import (ExpandStaticURLMixin,
                                    XBlockWithTranslationServiceMixin)

loader = ResourceLoader(__name__)


# Make '_' a no-op so we can scrape strings
def _(text):
    return text

# Classes ###########################################################


@XBlock.needs("i18n")
class TipBlock(StudioEditableXBlockMixin, XBlockWithTranslationServiceMixin, XBlock, ExpandStaticURLMixin):
    """
    Each choice can define a tip depending on selection
    """
    content = String(
        display_name=_("Content"),
        help=_("Text of the tip to show if the student chooses this tip's associated choice[s]"),
        scope=Scope.content,
        default=""
    )
    values = List(
        display_name=_("For Choices"),
        help=_("List of choices for which to display this tip"),
        scope=Scope.content,
        default=[],
        list_values_provider=lambda self: self.get_parent().human_readable_choices,
        list_style='set',  # Underered, unique items. Affects the UI editor.
    )
    width = String(
        display_name=_("Width"),
        help=_("Width of the tip popup (e.g. '400px')"),
        scope=Scope.content,
        default=''
    )
    height = String(
        display_name=_("Height"),
        help=_("Height of the tip popup (e.g. '200px')"),
        scope=Scope.content,
        default=''
    )
    editable_fields = ('values', 'content', 'width', 'height')

    @property
    def display_name_with_default(self):
        values_list = []
        for entry in self.get_parent().human_readable_choices:
            if entry["value"] in self.values:
                display_name = strip_tags(entry["display_name"])  # Studio studio_view can't handle html in display_name
                if len(display_name) > 20:
                    display_name = display_name[:20] + u'…'
                values_list.append(display_name)
        return self._(u"Tip for {list_of_choices}").format(list_of_choices=u", ".join(values_list))

    def mentoring_view(self, context=None):
        """ Render this XBlock within a mentoring block. """
        html = loader.render_template("templates/html/tip.html", {
            'content': self.content,
            'width': self.width,
            'height': self.height,
        })
        return Fragment(html)

    def student_view_data(self, context=None):
        return {
            'display_name': self.display_name_with_default,
            'content': self.expand_static_url(self.content),
            'for_choices': self.values,
        }

    def student_view(self, context=None):
        """ Normal view of this XBlock, identical to mentoring_view """
        return self.mentoring_view(context)

    def clean_studio_edits(self, data):
        """
        Clean up the edits during studio_view save
        """
        if "values" in data:
            data["values"] = list([six.text_type(v) for v in set(data["values"])])

    def validate_field_data(self, validation, data):
        """
        Validate this block's field data.
        """
        super(TipBlock, self).validate_field_data(validation, data)

        def add_error(msg):
            validation.add(ValidationMessage(ValidationMessage.ERROR, msg))

        try:
            valid_values = set(self.get_parent().all_choice_values)
        except Exception:
            pass
        else:
            for dummy in set(data.values) - valid_values:
                add_error(self._(u"A choice selected for this tip does not exist."))

    @classmethod
    def parse_xml(cls, node, runtime, keys, id_generator):
        """
        Construct this XBlock from the given XML node.
        """
        block = runtime.construct_xblock_from_class(cls, keys)

        block.values = cls.values.from_string(node.get('values', '[]'))
        block.width = node.get('width', '')
        block.height = node.get('height', '')

        block.content = six.text_type(node.text or u"")
        for child in node:
            block.content += etree.tostring(child, encoding='unicode')

        return block



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#
"""
Each class in this file represents a change made to the XML schema between v1 and v2.
"""
import json
import warnings

import six
from lxml import etree


class Change(object):
    @staticmethod
    def applies_to(node):
        """
        Does this change affect the given XML node?
        n.b. prior Changes will already be applied to the node.
        """
        raise NotImplementedError

    def __init__(self, node):
        """
        Prepare to upgrade 'node' at some point in the future
        """
        self.node = node

    def apply(self):
        raise NotImplementedError


class RenameMentoringTag(Change):
    """
    Replace <mentoring> with <problem-builder>
    """
    @staticmethod
    def applies_to(node):
        return node.tag == "mentoring"

    def apply(self):
        self.node.tag = "problem-builder"


class PrefixTags(Change):
    """ Most old elements now get a pb- prefix """
    @staticmethod
    def applies_to(node):
        return node.tag in ("tip", "choice", "answer", "mcq", "mrq", "rating", "column", "message")

    def apply(self):
        self.node.tag = "pb-" + self.node.tag


class HideTitle(Change):
    """
    If no <title> element is present, set hide_title="true"
    """
    @staticmethod
    def applies_to(node):
        return node.tag == "problem-builder" and node.find("title") is None

    def apply(self):
        self.node.attrib["show_title"] = "false"


class RemoveTitle(Change):
    """ The old <title> element is now an attribute of <problem-builder> """
    @staticmethod
    def applies_to(node):
        return node.tag == "title" and node.getparent().tag == "problem-builder"

    def apply(self):
        title = self.node.text.strip() if self.node.text else u''
        p = self.node.getparent()
        old_display_name = p.get("display_name")
        if old_display_name and old_display_name != title:
            warnings.warn(
                u'Replacing display_name="{}" with <title> value "{}"'.format(p.attrib["display_name"], title)
            )
        p.attrib["display_name"] = title
        p.remove(self.node)


class UnwrapHTML(Change):
    """ <pb-choice>,<pb-tip>, <header>, and <pb-message> now contain HTML without an explicit <html> wrapper. """
    @staticmethod
    def applies_to(node):
        return node.tag == "html" and node.getparent().tag in ("pb-choice", "pb-tip", "pb-message", "header")

    def apply(self):
        p = self.node.getparent()
        if self.node.text:
            p.text = (p.text if p.text else u"") + self.node.text
        index = list(p).index(self.node)
        for child in list(self.node):
            index += 1
            p.insert(index, child)
        p.remove(self.node)


class RenameTableTag(Change):
    """
    Replace <mentoring-table> with <pb-table>
    """
    @staticmethod
    def applies_to(node):
        return node.tag == "mentoring-table"

    def apply(self):
        self.node.tag = "pb-table"


class TableColumnHeader(Change):
    """
    Replace:
    <pb-table>
         <pb-column>
             <header>Answer 1</header>
             <answer name="answer_1" />
         </pb-column>
    </pb-table>
    with
    <pb-table>
         <pb-column header="Answer 1">
             <answer-recap name="answer_1" />
         </pb-column>
    </pb-table>
    """
    @staticmethod
    def applies_to(node):
        return node.tag == "pb-column" and node.getparent().tag == "pb-table"

    def apply(self):
        header_html = u""
        to_remove = []
        for child in list(self.node):
            if child.tag == "header":
                if child.text:
                    header_html += child.text
                for grandchild in list(child):
                    header_html += etree.tostring(grandchild).decode('utf-8')
                to_remove.append(child)
            elif child.tag == "pb-answer":
                child.tag = "pb-answer-recap"
                if "read_only" in child.attrib:
                    del child.attrib["read_only"]
            elif child.tag != "html":
                warnings.warn("Invalid <pb-column> element: Unexpected <{}>".format(child.tag))
                return
        for child in to_remove:
            self.node.remove(child)
        self.node.text = None
        if header_html:
            self.node.attrib["header"] = header_html


class QuizzToMCQ(Change):
    """ <quizz> element was an alias for <mcq>. In v2 we only have <pb-mcq> """
    @staticmethod
    def applies_to(node):
        return node.tag == "quizz"

    def apply(self):
        self.node.tag = "pb-mcq"


class MCQToRating(Change):
    """ <mcq type="rating"> is now just <rating>, and we never use type="choices" on MCQ/MRQ """
    @staticmethod
    def applies_to(node):
        return node.tag in ("pb-mcq", "pb-mrq") and "type" in node.attrib

    def apply(self):
        if self.node.tag == "pb-mcq" and self.node.get("type") == "rating":
            self.node.tag = "pb-rating"
        self.node.attrib.pop("type")  # Type attribute is no longer used.


class ReadOnlyAnswerToRecap(Change):
    """ <answer read_only="true"> is now <pb-answer-recap/> """
    @staticmethod
    def applies_to(node):
        return node.tag == "pb-answer" and node.get("read_only") == "true"

    def apply(self):
        self.node.tag = "pb-answer-recap"
        self.node.attrib.pop("read_only")
        for name in self.node.attrib:
            if name != "name":
                warnings.warn("Invalid attribute found on <answer>: {}".format(name))


class QuestionToField(Change):
    """
    <answer/mcq/mrq/rating>
        <question>What is the answer?</question>
    </answer/mcq/mrq/rating>

    has become

    <answer/mcq/mrq question="What is the answer?"></answer>
    """
    @staticmethod
    def applies_to(node):
        parent_tags = ("pb-answer", "pb-mcq", "pb-mrq", "pb-rating")
        return node.tag == "question" and node.getparent().tag in parent_tags

    def apply(self):
        if list(self.node):
            warnings.warn("Ignoring unexpected children of a <question> element. HTML may be lost.")
        p = self.node.getparent()
        p.attrib["question"] = self.node.text
        p.remove(self.node)


class QuestionSubmitMessageToField(Change):
    """
    <mcq/mrq>
        <message type="on-submit">Thank you for answering!</message>
    </mcq/mrq>

    has become

    <mcq/mrq message="Thank you for answering!"></answer>
    """
    @staticmethod
    def applies_to(node):
        return (
            node.tag == "pb-message" and
            node.get("type") == "on-submit" and
            node.getparent().tag in ("pb-mcq", "pb-mrq")
        )

    def apply(self):
        if list(self.node):
            warnings.warn("Ignoring unexpected children of a <message> element. HTML may be lost.")
        p = self.node.getparent()
        p.attrib["message"] = self.node.text
        p.remove(self.node)


class TipChanges(Change):
    """
    Changes to <tip></tip> elements.
    The main one being that the correctness of each choice is now stored on the MRQ/MCQ block, not on the <tip>s.
    """
    @staticmethod
    def applies_to(node):
        return node.tag == "pb-tip" and node.getparent().tag in ("pb-mcq", "pb-mrq", "pb-rating")

    def apply(self):
        p = self.node.getparent()

        def add_to_list(list_name, value):
            if list_name in p.attrib:
                p.attrib[list_name] += ",{}".format(value)
            else:
                p.attrib[list_name] = value

        if p.tag == "pb-mrq":
            if self.node.attrib.get("display"):
                value = self.node.attrib.pop("display")
                add_to_list("ignored_choices", value)
            elif self.node.attrib.get("require"):
                value = self.node.attrib.pop("require")
                add_to_list("required_choices", value)
            elif self.node.attrib.get("reject"):
                value = self.node.attrib.pop("reject")
            else:
                warnings.warn(u"Invalid <tip> element found: {}".format(etree.tostring(self.node)))
                return
        else:
            # This is an MCQ or Rating question:
            if self.node.attrib.get("display"):
                value = self.node.attrib.pop("display")
                add_to_list("correct_choices", value)
            elif self.node.attrib.get("reject"):
                value = self.node.attrib.pop("reject")
            elif self.node.attrib.get("require"):
                value = self.node.attrib.pop("require")
                add_to_list("correct_choices", value)
                warnings.warn(u"<tip> element in an MCQ/Rating used 'require' rather than 'display'")
            else:
                warnings.warn(u"Invalid <tip> element found: {}".format(etree.tostring(self.node)))
                return
        self.node.attrib["values"] = value
        if (self.node.text is None or self.node.text.strip() == "") and not list(self.node):
            # This tip is blank.
            p.remove(self.node)


class SharedHeaderToHTML(Change):
    """ <shared-header> element no longer exists. Just use <html> """
    @staticmethod
    def applies_to(node):
        return node.tag == "shared-header" and node.getparent().tag == "problem-builder"

    def apply(self):
        self.node.tag = "html"


class CommaSeparatedListToJson(Change):
    APPLY_TO_ATTRIBUTES = ("values", "correct_choices", "required_choices", "ignored_choices")

    def _convert_value(self, raw_value):
        return json.dumps([six.text_type(val).strip() for val in raw_value.split(',')])

    @staticmethod
    def applies_to(node):
        return node.tag in ("pb-tip", "pb-mrq", "pb-mcq", "pb-rating")

    def apply(self):
        for attribute in self.APPLY_TO_ATTRIBUTES:
            if attribute in self.node.attrib:
                self.node.attrib[attribute] = self._convert_value(self.node.attrib[attribute])


class OptionalShowTitleDefaultToFalse(Change):
    """
    In recent versions of mentoring, show_title defaults to True. In old versions there were no
    titles. If upgrading an old version, show_title should be set False.
    """
    @staticmethod
    def applies_to(node):
        return node.tag in ("pb-answer", "pb-mrq", "pb-mcq", "pb-rating") and ("show_title" not in node.attrib)

    def apply(self):
        self.node.attrib["show_title"] = "false"


def convert_xml_to_v2(node, from_version="v1"):
    """
    Given an XML node, re-structure it as needed to convert it from v1 style to v2 style XML.

    If from_version is set to "v0", then the "show_title" attribute on each question will be set
    to False, for compatibility with old versions of the mentoring block that didn't have
    question titles at all.
    """

    # An *ordered* list of all XML schema changes:
    xml_changes = [
        RenameMentoringTag,
        PrefixTags,
        HideTitle,
        RemoveTitle,
        UnwrapHTML,
        RenameTableTag,
        TableColumnHeader,
        QuizzToMCQ,
        MCQToRating,
        ReadOnlyAnswerToRecap,
        QuestionToField,
        QuestionSubmitMessageToField,
        TipChanges,
        SharedHeaderToHTML,
        CommaSeparatedListToJson,
    ]

    if from_version == "v0":
        xml_changes.append(OptionalShowTitleDefaultToFalse)

    # Apply each individual type of change one at a time:
    for change in xml_changes:
        # Walk the XML tree once and figure out all the changes we will need.
        # This lets us avoid modifying the tree while walking it.
        changes_needed = []
        for element in node.iter():
            if change.applies_to(element):
                changes_needed.append(change(element))
        for change in changes_needed:
            change.apply()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#
import ddt
import mock
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.support.ui import WebDriverWait

from .base_test import MentoringBaseTest, ProblemBuilderBaseTest


class MentoringTest(MentoringBaseTest):
    def test_display_submit_false_does_not_display_submit(self):
        mentoring = self.go_to_page('No Display Submit')
        with self.assertRaises(NoSuchElementException):
            mentoring.find_element_by_css_selector('.submit input.input-main')


def _get_mentoring_theme_settings(theme):
    return {
        'package': 'problem_builder',
        'locations': ['public/themes/{}.css'.format(theme)]
    }


@ddt.ddt
class ProblemBuilderQuestionnaireBlockTest(ProblemBuilderBaseTest):
    def _get_xblock(self, mentoring, name):
        return mentoring.find_element_by_css_selector(".xblock-v1[data-name='{}']".format(name))

    def _get_choice(self, questionnaire, choice_index):
        return questionnaire.find_elements_by_css_selector(".choices-list .choice")[choice_index]

    def _get_answer_checkmark(self, answer):
        return answer.find_element_by_xpath("ancestor::node()[3]").find_element_by_css_selector(".answer-checkmark")

    def _get_completion_checkmark(self, completion):
        return completion.find_element_by_xpath("ancestor::node()[4]").find_element_by_css_selector(".submit-result")

    def _get_messages_element(self, mentoring):
        return mentoring.find_element_by_css_selector('.messages')

    def _get_submit(self, mentoring):
        return mentoring.find_element_by_css_selector('.submit input.input-main')

    def _get_controls(self, mentoring):
        answer = self._get_xblock(mentoring, "feedback_answer_1").find_element_by_css_selector('.answer')
        mcq = self._get_xblock(mentoring, "feedback_mcq_2")
        mrq = self._get_xblock(mentoring, "feedback_mrq_3")
        rating = self._get_xblock(mentoring, "feedback_rating_4")
        completion = self._get_xblock(mentoring, "completion_1").find_element_by_css_selector('.pb-completion-value')

        return answer, mcq, mrq, rating, completion

    def _assert_answer(self, answer, results_shown=True):
        self.assertEqual(answer.get_attribute('value'), 'This is the answer')
        answer_checkmark = self._get_answer_checkmark(answer)
        self._assert_checkmark(answer_checkmark, shown=results_shown)

    def _assert_completion(self, completion, results_shown=True):
        self.assertTrue(completion.is_selected())
        completion_checkmark = self._get_completion_checkmark(completion)
        if results_shown:
            self.assertTrue(completion_checkmark.is_displayed())
        else:
            self.assertFalse(completion_checkmark.is_displayed())

    def _assert_checkmark(self, checkmark, correct=True, shown=True):
        result_classes = checkmark.get_attribute('class').split()
        result_label = checkmark.get_attribute('aria-label').strip()
        if shown:
            if correct:
                checkmark_class = 'checkmark-correct'
                checkmark_label = 'Correct'
            else:
                checkmark_class = 'checkmark-incorrect'
                checkmark_label = 'Incorrect'

            self.assertTrue(checkmark.is_displayed())
            self.assertIn(checkmark_class, result_classes)
            self.assertEquals(checkmark_label, result_label)
        else:
            self.assertFalse(checkmark.is_displayed())
            self.assertEquals('', result_label)

    def _assert_mcq(self, mcq, previous_answer_shown=True):
        if previous_answer_shown:
            self._assert_feedback_shown(mcq, 0, "Great!", click_choice_result=True)
        else:
            for i in range(3):
                self._assert_feedback_hidden(mcq, i)
                self._assert_not_checked(mcq, i)

    def _assert_rating(self, rating, previous_answer_shown=True):
        if previous_answer_shown:
            self._assert_feedback_shown(rating, 3, "I love good grades.", click_choice_result=True)
        else:
            for i in range(5):
                self._assert_feedback_hidden(rating, i)
                self._assert_not_checked(rating, i)

    def _assert_feedback_shown(
            self, questionnaire, choice_index, expected_text, click_choice_result=False, success=True
    ):
        """
        Asserts that feedback for given element contains particular text
        If `click_choice_result` is True - clicks on `choice-result` icon before checking feedback visibility:
        MRQ feedbacks are not shown right away
        """
        choice = self._get_choice(questionnaire, choice_index)
        choice_result = choice.find_element_by_css_selector('.choice-result')
        if click_choice_result:
            self.wait_until_clickable(choice_result)
            choice_result.click()

        feedback_popup = choice.find_element_by_css_selector(".choice-tips")
        self._assert_checkmark(choice_result, correct=success)
        self.assertTrue(feedback_popup.is_displayed())
        self.assertEqual(feedback_popup.text, expected_text)

    def _assert_feedback_hidden(self, questionnaire, choice_index):
        choice = self._get_choice(questionnaire, choice_index)
        choice_result = choice.find_element_by_css_selector('.choice-result')
        choice_input = choice.find_element_by_css_selector('input')
        feedback_popup = choice.find_element_by_css_selector(".choice-tips")
        result_classes = choice_result.get_attribute('class').split()

        if choice_input.is_selected():
            self.assertTrue(choice_result.is_displayed())
        self.assertFalse(feedback_popup.is_displayed())
        self.assertNotIn('checkmark-correct', result_classes)
        self.assertNotIn('checkmark-incorrect', result_classes)

    def _assert_not_checked(self, questionnaire, choice_index):
        choice = self._get_choice(questionnaire, choice_index)
        choice_input = choice.find_element_by_css_selector('input')
        self.assertFalse(choice_input.is_selected())

    def _assert_mrq(self, mrq, previous_answer_shown=True):
        if previous_answer_shown:
            self._assert_feedback_shown(
                mrq, 0, "This is something everyone has to like about this MRQ",
                click_choice_result=True
            )
            self._assert_feedback_shown(
                mrq, 1, "This is something everyone has to like about beauty",
                click_choice_result=True, success=False
            )
            self._assert_feedback_shown(mrq, 2, "This MRQ is indeed very graceful", click_choice_result=True)
            self._assert_feedback_shown(mrq, 3, "Nah, there aren't any!", click_choice_result=True, success=False)
        else:
            for i in range(3):
                self._assert_feedback_hidden(mrq, i)
                self._assert_not_checked(mrq, i)

    def _assert_messages(self, messages, shown=True):
        if shown:
            self.assertTrue(messages.is_displayed())
            self.assertEqual(messages.text, "FEEDBACK\nNot done yet")
        else:
            self.assertFalse(messages.is_displayed())
            self.assertEqual(messages.text, "")

    def _standard_filling(self, answer, mcq, mrq, rating, completion):
        # Long answer
        answer.send_keys('This is the answer')
        # MCQ
        self.click_choice(mcq, "Yes")
        # MRQ: Select 1st, 3rd and 4th options
        # First three options are required, so we're making two mistakes:
        # - 2nd option should have been selected
        # - 4th option should *not* have been selected
        self.click_choice(mrq, "Its elegance")
        self.click_choice(mrq, "Its gracefulness")
        self.click_choice(mrq, "Its bugs")
        # Rating
        self.click_choice(rating, "4")
        # Completion - tick the checkbox
        completion.click()

    # mcq and rating can't be reset easily, but it's not required; listing them here to keep method signature similar
    def _clear_filling(self, answer, mcq, mrq, rating, completion):      # pylint: disable=unused-argument
        answer.clear()
        completion.click()
        for checkbox in mrq.find_elements_by_css_selector('.choice input'):
            if checkbox.is_selected():
                checkbox.click()

    def _standard_checks(self, answer, mcq, mrq, rating, completion, messages):
        self.wait_until_visible(messages)

        # Long answer: Previous answer and results visible
        self._assert_answer(answer)
        # MCQ: Previous answer and results visible
        self._assert_mcq(mcq)
        # MRQ: Previous answer and feedback visible
        self._assert_mrq(mrq)
        # Rating: Previous answer and results visible
        self._assert_rating(rating)
        # Completion: Previous answer and results visible
        self._assert_completion(completion)
        # Messages visible
        self._assert_messages(messages)

    def _mcq_hide_previous_answer_checks(self, answer, mcq, mrq, rating, completion, messages):
        self.wait_until_visible(messages)

        # Long answer: Previous answer and results visible
        self._assert_answer(answer)
        # MCQ: Previous answer and results hidden
        self._assert_mcq(mcq, previous_answer_shown=False)
        # MRQ: Previous answer and results hidden
        self._assert_mrq(mrq, previous_answer_shown=False)
        # Rating: Previous answer and results hidden
        self._assert_rating(rating, previous_answer_shown=False)
        # Completion: Previous answer and results visible
        self._assert_completion(completion)
        # Messages visible
        self._assert_messages(messages)

    def _hide_feedback_checks(self, answer, mcq, mrq, rating, completion, messages):
        # Long answer: Previous answer visible and results hidden
        self._assert_answer(answer, results_shown=False)
        # MCQ: Previous answer and results visible
        self._assert_mcq(mcq)
        # MRQ: Previous answer and results visible
        self._assert_mrq(mrq)
        # Rating: Previous answer and results visible
        self._assert_rating(rating)
        # Completion: Previous answer visible and results hidden
        self._assert_completion(completion, results_shown=False)
        # Messages hidden
        self._assert_messages(messages, shown=False)

    def _mcq_hide_previous_answer_hide_feedback_checks(self, answer, mcq, mrq, rating, completion, messages):
        # Long answer: Previous answer visible and results hidden
        self._assert_answer(answer, results_shown=False)
        # MCQ: Previous answer and results hidden
        self._assert_mcq(mcq, previous_answer_shown=False)
        # MRQ: Previous answer and results hidden
        self._assert_mrq(mrq, previous_answer_shown=False)
        # Rating: Previous answer and feedback hidden
        self._assert_rating(rating, previous_answer_shown=False)
        # Completion: Previous answer visible and results hidden
        self._assert_completion(completion, results_shown=False)
        # Messages hidden
        self._assert_messages(messages, shown=False)

    def reload_student_view(self):
        # Load another page (the home page), then go back to the page we want. This is the only reliable way to reload.
        self.browser.get(self.live_server_url + '/')
        wait = WebDriverWait(self.browser, self.timeout)

        def did_load_homepage(driver):
            title = driver.find_element_by_css_selector('h1.title')
            return title and title.text == "XBlock scenarios"

        wait.until(did_load_homepage, u"Workbench home page should have loaded")
        mentoring = self.go_to_view("student_view")
        submit = self._get_submit(mentoring)
        self.wait_until_visible(submit)
        return mentoring

    def test_feedback_and_messages_not_shown_on_first_load(self):
        mentoring = self.load_scenario("feedback_persistence.xml")
        answer, mcq, mrq, rating, completion = self._get_controls(mentoring)
        messages = self._get_messages_element(mentoring)
        submit = self._get_submit(mentoring)

        answer_checkmark = self._get_answer_checkmark(answer)
        self._assert_checkmark(answer_checkmark, shown=False)
        for i in range(3):
            self._assert_feedback_hidden(mcq, i)
        for i in range(4):
            self._assert_feedback_hidden(mrq, i)
        for i in range(5):
            self._assert_feedback_hidden(rating, i)
        completion_checkmark = self._get_completion_checkmark(completion)
        self.assertFalse(completion_checkmark.is_displayed())
        self.assertFalse(messages.is_displayed())
        self.assertFalse(submit.is_enabled())

    def test_persists_feedback_on_page_reload(self):
        options = {
            'pb_mcq_hide_previous_answer': False,
            'pb_hide_feedback_if_attempts_remain': False
        }
        self._test_persistence(options, "_standard_checks")

    def test_does_not_persist_feedback_on_page_reload_if_disabled(self):
        options = {
            'pb_mcq_hide_previous_answer': False,
            'pb_hide_feedback_if_attempts_remain': True
        }
        self._test_persistence(options, "_hide_feedback_checks")

    def test_does_not_persist_mcq_on_page_reload_if_disabled(self):
        options = {
            'pb_mcq_hide_previous_answer': True,
            'pb_hide_feedback_if_attempts_remain': False
        }
        self._test_persistence(options, "_mcq_hide_previous_answer_checks")

    def test_does_not_persist_mcq_and_feedback_on_page_reload_if_disabled(self):
        options = {
            'pb_mcq_hide_previous_answer': True,
            'pb_hide_feedback_if_attempts_remain': True
        }
        self._test_persistence(options, "_mcq_hide_previous_answer_hide_feedback_checks")

    def _test_persistence(self, options, after_reload_checks):
        with mock.patch("problem_builder.mentoring.MentoringBlock.get_options") as patched_options:
            patched_options.return_value = options
            mentoring = self.load_scenario("feedback_persistence.xml")
            answer, mcq, mrq, rating, completion = self._get_controls(mentoring)
            messages = self._get_messages_element(mentoring)

            self._standard_filling(answer, mcq, mrq, rating, completion)
            self.click_submit(mentoring)
            self._standard_checks(answer, mcq, mrq, rating, completion, messages)

            # Now reload the page...
            mentoring = self.reload_student_view()
            answer, mcq, mrq, rating, completion = self._get_controls(mentoring)
            messages = self._get_messages_element(mentoring)
            submit = self._get_submit(mentoring)

            # ... and see if previous answers, results, feedback are shown/hidden correctly
            getattr(self, after_reload_checks)(answer, mcq, mrq, rating, completion, messages)

            # After reloading, submit is enabled only when:
            #  - feedback is hidden; and
            #  - previous MCQ/MRQ answers are visible.
            # When feedback is visible there's no need to resubmit the same answer;
            # and when previous MCQ/MRQ answers are hidden, submit is disabled until you select some options.
            if options['pb_hide_feedback_if_attempts_remain'] and not options['pb_mcq_hide_previous_answer']:
                self.assertTrue(submit.is_enabled())
            else:
                self.assertFalse(submit.is_enabled())

            # When student makes changes, submit is enabled again.
            self.click_choice(mcq, "Maybe not")
            self.click_choice(mrq, "Its elegance")
            self.click_choice(rating, "2")
            self.assertTrue(submit.is_enabled())

    def test_given_perfect_score_in_past_loads_current_result(self):
        mentoring = self.load_scenario("feedback_persistence.xml")
        answer, mcq, mrq, rating, completion = self._get_controls(mentoring)
        messages = self._get_messages_element(mentoring)

        answer.send_keys('This is the answer')
        self.click_choice(mcq, "Yes")
        # 1st, 3rd and 4th options, first three are correct, i.e. two mistakes: 2nd and 4th
        self.click_choice(mrq, "Its elegance")
        self.click_choice(mrq, "Its gracefulness")
        self.click_choice(mrq, "Its beauty")
        self.click_choice(rating, "4")
        completion.click()
        self.click_submit(mentoring)

        # precondition - verifying 100% score achieved
        self.assertEqual(answer.get_attribute('value'), 'This is the answer')
        self._assert_feedback_shown(mcq, 0, "Great!", click_choice_result=True)
        self._assert_feedback_shown(
            mrq, 0, "This is something everyone has to like about this MRQ",
            click_choice_result=True
        )
        self._assert_feedback_shown(
            mrq, 1, "This is something everyone has to like about beauty",
            click_choice_result=True
        )
        self._assert_feedback_shown(mrq, 2, "This MRQ is indeed very graceful", click_choice_result=True)
        self._assert_feedback_shown(mrq, 3, "Nah, there aren't any!", click_choice_result=True)
        self._assert_feedback_shown(rating, 3, "I love good grades.", click_choice_result=True)
        self.assertTrue(completion.is_selected())
        self.assertTrue(messages.is_displayed())
        self.assertEqual(messages.text, "FEEDBACK\nAll Good")

        self._clear_filling(answer, mcq, mrq, rating, completion)
        self._standard_filling(answer, mcq, mrq, rating, completion)
        self.click_submit(mentoring)
        self._standard_checks(answer, mcq, mrq, rating, completion, messages)

        # now, reload the page and make sure LATEST submission is loaded and feedback is shown
        mentoring = self.reload_student_view()
        answer, mcq, mrq, rating, completion = self._get_controls(mentoring)
        messages = self._get_messages_element(mentoring)
        self._standard_checks(answer, mcq, mrq, rating, completion, messages)

    def test_partial_mrq_is_not_completed(self):
        mentoring = self.load_scenario("feedback_persistence.xml")
        answer, mcq, mrq, rating, completion = self._get_controls(mentoring)
        messages = self._get_messages_element(mentoring)

        answer.send_keys('This is the answer')
        self.click_choice(mcq, "Yes")
        # 1st, 3rd and 4th options, first three are correct, i.e. two mistakes: 2nd and 4th
        self.click_choice(mrq, "Its elegance")
        self.click_choice(mrq, "Its gracefulness")
        self.click_choice(rating, "4")
        completion.click()
        self.click_submit(mentoring)

        def assert_state(answer, mcq, mrq, rating, completion, messages):
            self.wait_until_visible(messages)
            self.assertEqual(answer.get_attribute('value'), 'This is the answer')
            self._assert_feedback_shown(mcq, 0, "Great!", click_choice_result=True)
            self._assert_feedback_shown(
                mrq, 0, "This is something everyone has to like about this MRQ",
                click_choice_result=True
            )
            self._assert_feedback_shown(
                mrq, 1, "This is something everyone has to like about beauty",
                click_choice_result=True, success=False
            )
            self._assert_feedback_shown(mrq, 2, "This MRQ is indeed very graceful", click_choice_result=True)
            self._assert_feedback_shown(mrq, 3, "Nah, there aren't any!", click_choice_result=True)
            self._assert_feedback_shown(rating, 3, "I love good grades.", click_choice_result=True)
            self.assertTrue(completion.is_selected())
            self.assertTrue(messages.is_displayed())
            self.assertEqual(messages.text, "FEEDBACK\nNot done yet")

        assert_state(answer, mcq, mrq, rating, completion, messages)

        # now, reload the page and make sure the same result is shown
        mentoring = self.reload_student_view()
        answer, mcq, mrq, rating, completion = self._get_controls(mentoring)
        messages = self._get_messages_element(mentoring)
        assert_state(answer, mcq, mrq, rating, completion, messages)

    @ddt.unpack
    @ddt.data(
        # Questionnaire with tips.
        ("feedback_persistence_mcq_tips.xml", 'MCQ', True, False),
        ("feedback_persistence_mrq_tips.xml", 'MRQ', True, False),
        # Like the above but instead of tips in MCQ/MRQ
        # has a question level feedback. This feedback should also be suppressed.
        ("feedback_persistence_mcq_general_feedback.xml", 'MCQ', False, True),
        ("feedback_persistence_mrq_general_feedback.xml", 'MRQ', False, True),
        # These examples have both the choice tips and the general feedback.
        ("feedback_persistence_mcq_general_feedback_and_tips.xml", 'MCQ', True, True),
        ("feedback_persistence_mrq_general_feedback_and_tips.xml", 'MRQ', True, True),
        # And these have neither the tips nor the general feedback.
        ("feedback_persistence_mcq_no_feedback.xml", 'MCQ', False, False),
        ("feedback_persistence_mrq_no_feedback.xml", 'MRQ', False, False),
    )
    def test_feedback_persistence_tips(self, scenario, question_type, has_tips, has_feedback):
        tips_selector = '.choice-tips'
        feedback_selector = '.feedback'
        # Tests whether feedback (global message and choice tips) is hidden on reload.
        with mock.patch("problem_builder.mentoring.MentoringBlock.get_options") as patched_options:
            patched_options.return_value = {'pb_mcq_hide_previous_answer': True}
            mentoring = self.load_scenario(scenario)
            questionnaire = self._get_xblock(mentoring, "feedback_questionnaire")
            choice = self._get_choice(questionnaire, 0)
            choice_label = choice.find_element_by_css_selector('label')
            choice_result = choice.find_element_by_css_selector('.choice-result')
            tips = mentoring.find_element_by_css_selector(tips_selector)
            feedback = mentoring.find_element_by_css_selector(feedback_selector)

            self.assertFalse(tips.is_displayed())
            self.assertFalse(feedback.is_displayed())

            # Select choice.
            choice_label.click()
            self.click_submit(mentoring)

            # If the question has no general feedback and no tips,
            # nothing should be displayed.
            if not has_feedback and not has_tips:
                self.assertFalse(feedback.is_displayed())
                self.assertFalse(tips.is_displayed())
            # If there is general feedback, but no choice tips,
            # general feedback is displayed after submitting the answer.
            if has_feedback and not has_tips:
                self.assertTrue(feedback.is_displayed())
                self.assertFalse(tips.is_displayed())
            # If there are tips but no feedback, the result depends on question type:
            # - the tip is displayed by MCQs
            # - nothing is displayed by MRQs
            if has_tips and not has_feedback:
                self.assertFalse(feedback.is_displayed())
                if question_type == 'MCQ':
                    self.assertTrue(tips.is_displayed())
                elif question_type == 'MRQ':
                    self.assertFalse(tips.is_displayed())
            # If there are both tips and general feedback,
            # the result depends on the question type:
            # - the tip is displayed by MCQs
            # - general feedback is displayed by MRQs.
            if has_feedback and has_tips:
                if question_type == 'MCQ':
                    self.assertTrue(tips.is_displayed())
                    self.assertFalse(feedback.is_displayed())
                elif question_type == 'MRQ':
                    self.assertTrue(feedback.is_displayed())
                    self.assertFalse(tips.is_displayed())

            # Click the result icon; this hides the general feedback and displays
            # the choice tip, if present.
            choice_result.click()

            self.assertFalse(feedback.is_displayed())
            if has_tips:
                self.assertTrue(tips.is_displayed())

            # Reload the page.
            mentoring = self.reload_student_view()
            tips = mentoring.find_element_by_css_selector(tips_selector)
            feedback = mentoring.find_element_by_css_selector(feedback_selector)
            # After reloading the page, neither the tips nor the feedback should be displayed.
            self.assertFalse(tips.is_displayed())
            self.assertFalse(feedback.is_displayed())

    @ddt.data(
        (0, "Yes", "Great!"),
        (1, "Maybe not", "Ah, damn."),
        (2, "I don't understand", "Really?")
    )
    @ddt.unpack
    def test_clicking_result_icon_shows_tips(self, choice_index, choice_text, expected_tip):
        mentoring = self.load_scenario("feedback_persistence_mcq_tips.xml")
        mcq = self._get_xblock(mentoring, "feedback_questionnaire")
        question_title = mentoring.find_element_by_css_selector('.question-title')
        choice = self._get_choice(mcq, choice_index)
        choice_result = choice.find_element_by_css_selector('.choice-result')
        choice_tips = choice.find_element_by_css_selector('.choice-tips')

        def assert_tip():
            self.wait_until_visible(choice_tips)
            tip = choice_tips.find_element_by_css_selector('.tip')
            self.assertEqual(tip.text, expected_tip)

        # Answer question
        self.click_choice(mcq, choice_text)
        # Submit answer
        self.click_submit(mentoring)
        # Tip for selected choice should be visible and match expected tip
        assert_tip()
        # Random click to make tip disappear
        question_title.click()
        # Tip for selected choice should be hidden
        self.wait_until_hidden(choice_tips)
        # Click result icon to show tips
        choice_result.click()
        # Tip for selected choice should be visible and match expected tip
        assert_tip()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#
"""
Visual Representation of Dashboard State.

Consists of a series of images, layered on top of each other, where the appearance of each layer
can be tied to the average value of the student's response to a particular Problem Builder
block.

For example, each layer can have its color turn green if the student's average value on MCQs in
a specific Problem Builder block was at least 3.
"""


class DashboardVisualData(object):
    """
    Data about the visual representation of a dashboard.
    """
    def __init__(self, blocks, rules, color_for_value, title, desc):
        """
        Construct the data required for the optional visual representation of the dashboard.

        Data format accepted for rules is like:
        {
            "images": [
                "/static/step1.png",
                "/static/step2.png",
                "/static/step3.png",
                "/static/step4.png",
                "/static/step5.png",
                "/static/step6.png",
                "/static/step7.png"
            ],
            "background": "/static/background.png",
            "overlay": "/static/overlay.png",
            "width": "500",
            "height": "500"
        }

        color_for_value is a method that, given a value, returns a color string or None
        """
        # Images is a list of images, one per PB block, in the same order as 'blocks'
        # All images are rendered layered on top of each other, and can be hidden,
        # shown, colorized, faded, etc. based on the average answer value for that PB block.
        images = rules.get("images", [])
        # Overlay is an optional images drawn on top, with no effects applied
        overlay = rules.get("overlay")
        # Background is an optional images drawn on the bottom, with no effects applied
        background = rules.get("background")
        # Width and height of the image:
        self.width = int(rules.get("width", 400))
        self.height = int(rules.get("height", 400))
        # A unique ID used by the HTML template:
        self.unique_id = id(self)
        # Alternate dsescription for screen reader users:
        self.title = title
        self.desc = desc

        self.layers = []
        if background:
            self.layers.append({"url": background})
        for idx, block in enumerate(blocks):
            if not block.get("has_average"):
                continue  # We only use blocks with numeric averages for the visual representation
            # Now we build the 'layer_data' information to pass on to the template:
            try:
                layer_data = {"url": images[idx], "id": "layer{}".format(idx)}
            except IndexError:
                break

            # Check if a color rule applies:
            layer_data["color"] = color_for_value(block["average"])
            self.layers.append(layer_data)

        if overlay:
            self.layers.append({"url": overlay})
